#include "client_starter.h"

extern int perform_Attestation(int a)
{
	config_t config;
	sgx_launch_token_t token= { 0 };
	sgx_enclave_id_t eid= a;
	sgx_status_t status, sgxrv, pse_status;
	sgx_ra_msg1_t msg1;
	sgx_ra_msg2_t *msg2 = NULL;
	sgx_ra_msg3_t *msg3 = NULL;
	ra_msg4_t *msg4 = NULL;
	uint32_t msg0_extended_epid_group_id = 0;
	uint32_t msg3_sz;
	uint32_t flags= config.flags;
	sgx_ra_context_t ra_ctx= 0xdeadbeef;
	int rv;
	MsgIO *msgio;
	size_t msg4sz = 0;
	int enclaveTrusted = NotTrusted; // Not Trusted
	int b_pse= OPT_ISSET(flags, OPT_PSE);

	int result_operation=-1;
	memset(&config, 0, sizeof(config));
	config.mode= MODE_ATTEST;
	int opt_index= 0;
    	/* Configuration of the server */
	config.server= strdup("localhost");
	if ( config.server == NULL ) {
		perror("malloc");
		return 1;
	}

	if ( config.server == NULL ) {
		msgio = new MsgIO();
	} else {
		try {
			
			msgio = new MsgIO(config.server, (config.port == 0) ?
				"7777" : config.port);
		}
		catch(const std::exception& e) {
			std::cout << "Caught exception \"" << e.what() << "\"\n";
			fprintf(stderr,"Error during creation of msgio\n");
			exit(1);
		}
	}

	/*
	 * WARNING! Normally, the public key would be hardcoded into the
	 * enclave, not passed in as a parameter. Hardcoding prevents
	 * the enclave using an unauthorized key.
	 *
	 * This is diagnostic/test application, however, so we have
	 * the flexibility of a dynamically assigned key.
	 */

	/* Executes an ECALL that runs sgx_ra_init() */

	if ( OPT_ISSET(flags, OPT_PUBKEY) ) {
		
		[ENCLAVES_RA_INIT_SWITCHES]
	} else {
		
        [ENCLAVES_RA_INIT_DEF_SWITCHES]
	}

	/* Did the ECALL succeed? */
	if ( status != SGX_SUCCESS ) {
		fprintf(stderr, "enclave_ra_init: %08x\n", status);
		//delete msgio;
		return 1;
	}

	/* Did sgx_ra_init() succeed? */
	if ( sgxrv != SGX_SUCCESS ) {
		fprintf(stderr, "sgx_ra_init: %08x\n", sgxrv);
		//delete msgio;
		return 1;
	}

	/* Generate msg0 */

	status = sgx_get_extended_epid_group_id(&msg0_extended_epid_group_id);
	if ( status != SGX_SUCCESS ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		
		fprintf(stderr, "sgx_get_extended_epid_group_id: %08x\n", status);
		//delete msgio;
		return 1;
	}
 
	/* Generate msg1 */
    
    /* The code generated must be the following:
     * if (eid == X)
     *     status = sgx_ra_get_msg1(ra_ctx,eid,enclave1_sgx_ra_get_ga,&msg16)
     * oppure
     *     sgx_ra_get_ga = enclaveX_sgx_ra_get_ga;
	 *	   status = sgx_ra_get_msg1(ra_ctx,eid,enclave1_sgx_ra_get_ga,&msg1)
     * 
     * */
	[SWITCH_CASE_RA_GET_GA]
	status= sgx_ra_get_msg1(ra_ctx, eid, sgx_ra_get_ga, &msg1);
	if ( status != SGX_SUCCESS ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "sgx_ra_get_msg1: %08x\n", status);
		//delete msgio;
		return 1;
	}


	/*
	 * Send msg0 and msg1 concatenated together (msg0||msg1). We do
	 * this for efficiency, to eliminate an additional round-trip
	 * between client and server. The assumption here is that most
	 * clients have the correct extended_epid_group_id so it's
	 * a waste to send msg0 separately when the probability of a
	 * rejection is astronomically small.
	 *
	 * If it /is/ rejected, then the client has only wasted a tiny
	 * amount of time generating keys that won't be used.
	 */

	dividerWithText(stderr, "Copy/Paste Msg0||Msg1 Below to SP");
	msgio->send_partial(&msg0_extended_epid_group_id,
		sizeof(msg0_extended_epid_group_id));
	msgio->send(&msg1, sizeof(msg1));
	divider(stderr);
	
	fprintf(stderr, "Waiting for msg2\n");

	/* Read msg2 
	 *
	 * msg2 is variable length b/c it includes the revocation list at
	 * the end. msg2 is malloc'd in readZ_msg do free it when done.
	 */
	sleep(1);
	rv= msgio->read((void **) &msg2, NULL);
	if ( rv == 0 ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "protocol error reading msg2\n");
		//delete msgio;
		exit(1);
	} else if ( rv == -1 ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "system error occurred while reading msg2\n");
		//delete msgio;
		exit(1);
	}


	/* Process Msg2, Get Msg3  */
	/* object msg3 is malloc'd by SGX SDK, so remember to free when finished */

	msg3 = NULL;

		//same, sgx_ra_get_msg3_trusted = enclaveX_sgx_ra_get_msg3_trusted && sgx_ra_proc_msg2_trusted = enclaveX_sgx_ra_proc_msg2_trusted
		// status = sgx_ra_proc_msg2(ra_ctx,eid,)
		[ENCLAVES_SGX_RA_PROC_MSG2_SWITCHES]
		[ENCLAVES_SGX_RA_GET_MSG3_TRUSTED_SWITCHES]
		/*if(eid==2)
			status = sgx_ra_proc_msg2(ra_ctx, eid,
		enclave1_sgx_ra_proc_msg2_trusted, enclave1_sgx_ra_get_msg3_trusted, msg2, 
		sizeof(sgx_ra_msg2_t) + msg2->sig_rl_size, &msg3, &msg3_sz);
		else if(eid==3)
			status = sgx_ra_proc_msg2(ra_ctx, eid,
		enclave2_sgx_ra_proc_msg2_trusted, enclave2_sgx_ra_get_msg3_trusted, msg2, 
		sizeof(sgx_ra_msg2_t) + msg2->sig_rl_size, &msg3, &msg3_sz);
	
		*/
		status = sgx_ra_proc_msg2(ra_ctx, eid,sgx_ra_proc_msg2_trusted,sgx_ra_get_msg3_trusted, msg2, 
		sizeof(sgx_ra_msg2_t) + msg2->sig_rl_size, &msg3, &msg3_sz);
	free(msg2);

	if ( status != SGX_SUCCESS ) {		
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "sgx_ra_proc_msg2: %08x\n", status);

		//delete msgio;
		return 1;
	} 
	
	dividerWithText(stderr, "Copy/Paste Msg3 Below to SP");
	msgio->send(msg3, msg3_sz);
	divider(stderr);
	
	if ( msg3 ) {
		free(msg3);
		msg3 = NULL;
	}
 
	/* Read Msg4 provided by Service Provider, then process */
        
	rv= msgio->read((void **)&msg4, &msg4sz);
	if ( rv == 0 ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "protocol error reading msg4\n");
		//delete msgio;
		exit(1);
	} else if ( rv == -1 ) {
		[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
		fprintf(stderr, "system error occurred while reading msg4\n");
		//delete msgio;
		exit(1);
	}

	edividerWithText("Enclave Trust Status from Service Provider");

	enclaveTrusted= msg4->status;
	if ( enclaveTrusted == Trusted ) {
		eprintf("Enclave TRUSTED\n");
	}
	else if ( enclaveTrusted == NotTrusted ) {
		eprintf("Enclave NOT TRUSTED\n");
		return 10; //debug
	}
	else if ( enclaveTrusted == Trusted_ItsComplicated ) {
		// Trusted, but client may be untrusted in the future unless it
		// takes action.

		eprintf("Enclave Trust is TRUSTED and COMPLICATED. The client is out of date and\nmay not be trusted in the future depending on the service provider's  policy.\n");
	} else {
		// Not Trusted, but client may be able to take action to become
		// trusted.

		eprintf("Enclave Trust is NOT TRUSTED and COMPLICATED. The client is out of date.\n");
	}
	
	edivider();
	/* check to see if we have a PIB by comparing to empty PIB */
	sgx_platform_info_t emptyPIB;
	memset(&emptyPIB, 0, sizeof (sgx_platform_info_t));

	int retPibCmp = memcmp(&emptyPIB, (void *)(&msg4->platformInfoBlob), sizeof (sgx_platform_info_t));
	/* We have a PIB, so check to see if there are actions to take */
	sgx_update_info_bit_t update_info;
	sgx_status_t ret = sgx_report_attestation_status(&msg4->platformInfoBlob, enclaveTrusted, &update_info);

    /* Check to see if there is an update needed */
	if ( ret == SGX_ERROR_UPDATE_NEEDED ) {
		//edividerWithText("Platform Update Required");
		//eprintf("The following Platform Update(s) are required to bring this\n");
		//eprintf("platform's Trusted Computing Base (TCB) back into compliance:\n\n");
		if( update_info.pswUpdate ) {
				//eprintf("  * Intel SGX Platform Software needs to be updated to the latest version.\n");
			}

			if( update_info.csmeFwUpdate ) {
				//eprintf("  * The Intel Management Engine Firmware Needs to be Updated.  Contact your\n");
				//eprintf("    OEM for a BIOS Update.\n");
			}

			if( update_info.ucodeUpdate )  {
				//eprintf("  * The CPU Microcode needs to be updated.  Contact your OEM for a platform\n");
				//eprintf("    BIOS Update.\n");
			}                                           
			//eprintf("\n");
		}


	/*
	 * If the enclave is trusted, fetch a hash of the the MK and SK from
	 * the enclave to show proof of a shared secret with the service 
	 * provider.
	 */

	if ( enclaveTrusted == Trusted || enclaveTrusted ==Trusted_ItsComplicated ) {
		sgx_status_t key_status, sha_status;
		sgx_sha256_hash_t mkhash, skhash;

		// First the MK

		[ENCLAVES_GET_KEY_HASH_SWITCHES]
		
	}

	free (msg4);

	[ENCLAVES_ENCLAVE_RA_CLOSE_SWITCHES]
	//delete msgio;

	return 0;
}

/*
 * Search for the enclave file and then try and load it.
 */

#ifndef _WIN32
sgx_status_t sgx_create_enclave_search (const char *filename, const int edebug,
	sgx_launch_token_t *token, int *updated, sgx_enclave_id_t *eid,
	sgx_misc_attribute_t *attr)
{
	struct stat sb;
	char epath[PATH_MAX];	/* includes NULL */

	/* Is filename an absolute path? */

	if ( filename[0] == '/' ) 
		return sgx_create_enclave(filename, edebug, token, updated, eid, attr);

	/* Is the enclave in the current working directory? */

	if ( stat(filename, &sb) == 0 )
		return sgx_create_enclave(filename, edebug, token, updated, eid, attr);

	/* Search the paths in LD_LBRARY_PATH */

	if ( file_in_searchpath(filename, getenv("LD_LIBRARY_PATH"), epath, PATH_MAX) )
		return sgx_create_enclave(epath, edebug, token, updated, eid, attr);
		
	/* Search the paths in DT_RUNPATH */

	if ( file_in_searchpath(filename, getenv("DT_RUNPATH"), epath, PATH_MAX) )
		return sgx_create_enclave(epath, edebug, token, updated, eid, attr);

	/* Standard system library paths */

	if ( file_in_searchpath(filename, DEF_LIB_SEARCHPATH, epath, PATH_MAX) )
		return sgx_create_enclave(epath, edebug, token, updated, eid, attr);

	/*
	 * If we've made it this far then we don't know where else to look.
	 * Just call sgx_create_enclave() which assumes the enclave is in
	 * the current working directory. This is almost guaranteed to fail,
	 * but it will insure we are consistent about the error codes that
	 * get reported to the calling function.
	 */

	return sgx_create_enclave(filename, edebug, token, updated, eid, attr);
}

int file_in_searchpath (const char *file, const char *search, char *fullpath, 
	size_t len)
{
	char *p, *str;
	size_t rem;
	struct stat sb;

	if ( search == NULL ) return 0;
	if ( strlen(search) == 0 ) return 0;

	str= strdup(search);
	if ( str == NULL ) return 0;

	p= strtok(str, ":");
	while ( p != NULL) {
		size_t lp= strlen(p);

		if ( lp ) {

			strncpy(fullpath, p, len-1);
			rem= (len-1)-lp-1;
			fullpath[len-1]= 0;

			strncat(fullpath, "/", rem);
			--rem;

			strncat(fullpath, file, rem);

			if ( stat(fullpath, &sb) == 0 ) {
				free(str);
				return 1;
			}
		}

		p= strtok(NULL, ":");
	}

	free(str);

	return 0;
}

#endif
