/*

Copyright 2019 Intel Corporation

This software and the related documents are Intel copyrighted materials,
and your use of them is governed by the express license under which they
were provided to you (License). Unless the License provides otherwise,
you may not use, modify, copy, publish, distribute, disclose or transmit
this software or the related documents without Intel's prior written
permission.

This software and the related documents are provided as is, with no
express or implied warranties, other than those that are expressly stated
in the License.

*/

#include "server_interface.h"
#include <functional>
#include <thread>
#include <mutex>
#include <regex>

/* Need a global for the signal handler */
MsgIO *msgio = NULL;
std::mutex m;

void *attestStart(IAS_Connection *ias, config_t* config,list<string> policy_mrsigner,list<string> policy_mrenclaves,list<sgx_prod_id_t> policy_prodids)
{
    ra_session_t session;
    sgx_ra_msg1_t msg1;
    sgx_ra_msg2_t msg2;
    ra_msg4_t msg4;
    char *sigrl = NULL;

    memset(&session, 0, sizeof(ra_session_t));

    /* Read message 0 and 1, then generate message 2 */

    if (!process_msg01(msgio, ias, &msg1, &msg2, &sigrl, config,
                       &session))
    {

        eprintf("error processing msg1\n");
        goto disconnect;
    }

    /* Send message 2 */

    /*
     * sgx_ra_msg2_t is a struct with a flexible array member at the
     * end (defined as uint8_t sig_rl[]). We could go to all the
     * trouble of building a byte array large enough to hold the
     * entire struct and then cast it as (sgx_ra_msg2_t) but that's
     * a lot of work for no gain when we can just send the fixed
     * portion and the array portion by hand.
     */

    dividerWithText(stderr, "Copy/Paste Msg2 Below to Client");

    msgio->send_partial((void *)&msg2, sizeof(sgx_ra_msg2_t));

    msgio->send(sigrl, msg2.sig_rl_size);

    edivider();

    /* Read message 3, and generate message 4 */

    if (!process_msg3(msgio, ias, &msg1, &msg4, config, &session,policy_mrsigner,policy_mrenclaves,policy_prodids))
    {
        eprintf("error processing msg3\n");
        goto disconnect;
    }

disconnect:
    msgio->disconnect();

    return NULL;
}
int startServerIAS()
{
    char *sigrl = NULL;
    char *buffer_key = NULL;
    config_t config;
    int oops;
    IAS_Connection *ias = NULL;
    char *port = NULL;
#ifndef _WIN32
    struct sigaction sact;
#endif

    memset(&config,0,sizeof(config_t));

 
    /* Config defaults */
    char *eptr = NULL;
    unsigned long val;
    list<string> policy_mrsigner;
    list<string> policy_mrenclaves;
    list<sgx_prod_id_t> policy_prodids;
    if (!cert_load_file(&config.signing_ca, IAS_REPORT_CA_FILE_LINUX))
    {
        crypto_perror("cert_load_file");
        eprintf("Could not load IAS Signing Cert CA\n");
        return 1;
    }
    eprintf("IAS_Report_Signing_CA_File loaded.\n");
    config.store = cert_init_ca(config.signing_ca);
    if (config.store == NULL)
    {
        eprintf("could not initialize certificate store\n");
        return 1;
    }

    config.ca_bundle = strdup(DEFAULT_CA_BUNDLE_LINUX);
    if (config.ca_bundle == NULL)
    {
        perror("strdup");
        return 1;
    }
    eprintf("ca bundle of certificate loaded.\n");

    if (strlen(IAS_PRIMARY_SUBSCRIPTION_KEY) != IAS_SUBSCRIPTION_KEY_SIZE)
    {
        eprintf("IAS Subscription Key must be %d-byte hex string\n", IAS_SUBSCRIPTION_KEY_SIZE);
        return 1;
    }

    strncpy((char *)config.pri_subscription_key, IAS_PRIMARY_SUBSCRIPTION_KEY, IAS_SUBSCRIPTION_KEY_SIZE);
    eprintf("IAS Primary Subscription Key loaded.\n");

     /*
     * The parameters are set as constants in settings.h, but eventually they could be read from the single text file settings
     * calling a function like readFromFile(param_to_be_read),since the format of the file is <name_param>=<value_param>
     * like it has been done for the mrsigner value
     */
    if (strlen(IAS_PRIMARY_SUBSCRIPTION_KEY) != IAS_SUBSCRIPTION_KEY_SIZE)
    {
        eprintf("IAS Subscription Key must be %d-byte hex string\n", IAS_SUBSCRIPTION_KEY_SIZE);
        return 1;
    }

    strncpy((char *)config.pri_subscription_key, IAS_PRIMARY_SUBSCRIPTION_KEY, IAS_SUBSCRIPTION_KEY_SIZE);
    config.pri_subscription_key[IAS_SUBSCRIPTION_KEY_SIZE]='\0';
    eprintf("IAS Primary Subscription Key loaded.\n");

    /* Reading of the secondary subscription key */
    if (strlen(IAS_SECONDARY_SUBSCRIPTION_KEY) != IAS_SUBSCRIPTION_KEY_SIZE)
    {
        eprintf("IAS Secondary Subscription Key must be %d-byte hex string\n",
                IAS_SUBSCRIPTION_KEY_SIZE);
        return 1;
    }

    strncpy((char *)config.sec_subscription_key, IAS_SECONDARY_SUBSCRIPTION_KEY, IAS_SUBSCRIPTION_KEY_SIZE);
    config.sec_subscription_key[IAS_SUBSCRIPTION_KEY_SIZE]='\0';
    eprintf("IAS Secondary Subscription Key loaded.\n");

    ifstream file;
    file.open("server_material/policy");
    if (!file.is_open())
    {
        printf("Policy file can't be opened,returns.\n");
        return 1;
    }
    string line;
    string value_mrsigner;
	/* If the MRENCLAVE value has been setted to the specific debug value, the app will skip the test. If that it's not the case, run the sigstruct.sh script with the client enclave to insert
	   the digest hash of the enclave to attest inside the policy file to read.*/
    string value_mrenclave;
    string value_prodid;
    string value_isvsvn;
    string value_debug;
    while (!file.eof())
    {
        getline(file, line);
        if (!line.find("MRSIGNER="))
        {
            value_mrsigner = line.erase(0, 9);

            /*if (debug_info_messages)
                cout << value_mrsigner << endl;*/
        }
        if (!line.find("MRENCLAVE="))
        {
            value_mrenclave = line.erase(0, 10);
            /*if (debug_info_messages)
                cout << value_mrenclave << endl;*/
        }
        if (!line.find("PRODID="))
        {
            value_prodid = line.erase(0, 7);

            /*if (debug_info_messages)
                cout << value_prodid.c_str() << endl;*/
        }
        if (!line.find("MIN_ISVSVN="))
        {
            value_isvsvn = line.erase(0, 11);

            /*if (debug_info_messages)
                cout << value_isvsvn.c_str() << endl;*/
        }
        if (!line.find("ALLOW_DEBUG="))
        {
            value_debug = line.erase(0, 12);

            /*if (debug_info_messages)
                cout << value_debug.c_str() << endl;*/
            break;
        }
    }
    file.close();
      /* MRSIGNER values from policy file to config file*/
    
    string value_string = value_mrsigner.c_str();
    regex reg(",");
    sregex_token_iterator iter(value_string.begin(),value_string.end(),reg,-1);
    sregex_token_iterator end;
    vector<string> tokens(iter,end);
    int i = tokens.size();
    string vett[i];
    int j = 0;
    for(string &s: tokens){

        vett[j] = s;
        policy_mrsigner.push_back(vett[j++]);
    }
    
    tokens.clear();
    // MRENCLAVE is read now from the policy file
    void *mrenclave = static_cast<void *>(const_cast<char *>(value_mrenclave.c_str()));
    value_string = value_mrenclave.c_str();
    sregex_token_iterator iter_mrenc(value_mrenclave.begin(),value_mrenclave.end(),reg,-1);
    sregex_token_iterator end_mrenc;
    vector<string> tokens_mrenc(iter_mrenc,end_mrenc);
    i = tokens_mrenc.size();
    string vett2[i];
    j = 0;
    for(string &s: tokens_mrenc){
        vett2[j] = s;
        policy_mrenclaves.push_back(s);
        j++;
    }
    tokens_mrenc.clear();
    
    eptr = NULL;
    // PRODUCTION ID of the enclave is read now from the policy file
    value_string = value_prodid.c_str();
    stringstream ss(value_string);
    vector<string> result;
    while(ss.good()){
        string substr;
        getline(ss,substr,',');
        result.push_back(substr);
    }
    int len = result.size();
    auto it = result.begin();
    while(it!=result.end()){
        char* prodid = const_cast<char*>((*it).c_str());
        val = strtoul(prodid, &eptr, 10); // value_prodid is the value of the Product ID of the Enclave
        if (*eptr != '\0' || val > 0xFFFF)
        {
            eprintf("Product Id must be a positive integer <= 65535\n");
            return 1;
        }
        policy_prodids.push_back(val);
        it++;
    }

    if (strlen(SPID_SIGNER) < 32)
    {
        eprintf("SPID must be 32-byte hex string\n");
        return 1;
    }
    if (!from_hexstring((unsigned char *)&config.spid, (unsigned char *)SPID_SIGNER, 16))
    {
        eprintf("SPID must be 32-byte hex string\n");
        return 1 ;
    }
    eprintf("SPID loaded.\n");

    // MIN_ISVSVN is read from the policy file
    char *isv_svn = static_cast<char *>(const_cast<char *>(value_isvsvn.c_str()));
    eptr = NULL;
    val = strtoul(isv_svn, &eptr, 10);
    if (*eptr != '\0' || val > (unsigned long)0xFFFF)
    {
        eprintf("Minimum ISV SVN must be a positive integer <= 65535\n");
        return 1;
    }
    config.min_isvsvn = val;
    eprintf("Minimum ISV SVN set.\n");

    // Set the enclave debug flag according to the policy file
    char *debug_true = static_cast<char *>(const_cast<char *>(value_debug.c_str()));
    config.allow_debug_enclave = atoi(debug_true);

    config.quote_type = SGX_LINKABLE_SIGNATURE;
    config.apiver = 4;
    if (config.apiver < IAS_MIN_VERSION || config.apiver >
                                               IAS_MAX_VERSION)
    {

        eprintf("version must be between %d and %d\n",
                IAS_MIN_VERSION, IAS_MAX_VERSION);
        return 1;
    }

    /*
     * Use the hardcoded default key unless one is provided on the
     * command line. Most real-world services would hardcode the
     * key since the public half is also hardcoded into the enclave.
     */

    if (config.service_private_key == NULL)
    {
        config.service_private_key = key_private_from_bytes(def_service_private_key); // def_service_private_key
        if (config.service_private_key == NULL)
        {
            crypto_perror("key_private_from_bytes");
            return 1;
        }
    }

    port = strdup(DEFAULT_PORT);
    if (port == NULL)
    {
        perror("strdup");
        return 1;
    }

    /* Initialize out support libraries */

    crypto_init();

    /* Initialize our IAS request object */

    try
    {
        ias = new IAS_Connection(
            (IAS_ON_PRODUCTION) ? IAS_SERVER_PRODUCTION : IAS_SERVER_DEVELOPMENT,
            0,
            (char *)(config.pri_subscription_key),
            (char *)(config.sec_subscription_key));
    }
    catch (...)
    {
        oops = 1;
        eprintf("exception while creating IAS request object\n");
        return 1;
    }

    ias->proxy_mode(IAS_PROXY_NONE);

    if (config.user_agent != NULL)
    {
        if (!ias->agent(config.user_agent))
        {
            eprintf("%s: unknown user agent\n", config.user_agent);
            return 0;
        }
    }

    /*
     * Set the cert store for this connection. This is used for verifying
     * the IAS signing certificate, not the TLS connection with IAS (the
     * latter is handled using config.ca_bundle).
     */
    ias->cert_store(config.store);

    /*
     * Set the CA bundle for verifying the IAS server certificate used
     * for the TLS session. If this isn't set, then the user agent
     * will fall back to it's default.
     */
    if (strlen(config.ca_bundle))
        ias->ca_bundle(config.ca_bundle);

    /* Get our message IO object. */

    try
    {
        if (msgio == NULL)
            msgio = new MsgIO(NULL, (port == NULL) ? DEFAULT_PORT : port);
    }
    catch (...)
    {
        return 1;
    }

#ifndef _WIN32
    /*
     * Install some rudimentary signal handlers. We just want to make
     * sure we gracefully shutdown the listen socket before we exit
     * to avoid "address already in use" errors on startup.
     */

    sigemptyset(&sact.sa_mask);
    sact.sa_flags = 0;
    sact.sa_handler = &cleanup_and_exit;

    if (sigaction(SIGHUP, &sact, NULL) == -1)
        perror("sigaction: SIGHUP");
    if (sigaction(SIGINT, &sact, NULL) == -1)
        perror("sigaction: SIGHUP");
    if (sigaction(SIGTERM, &sact, NULL) == -1)
        perror("sigaction: SIGHUP");
    if (sigaction(SIGQUIT, &sact, NULL) == -1)
        perror("sigaction: SIGHUP");
#endif
    m.lock();
    while (msgio->server_loop())
    {

        // startRA(config,&ias,&msgio);

        std::thread t(attestStart, std::ref(ias),&config,policy_mrsigner,policy_mrenclaves,policy_prodids);
        t.join();
        m.unlock();
        break;
    }
    
    if(debug_server_info == 1)
    printf("Destroyed msgio.\n"); // DEBUG
    crypto_destroy();

    // delete msgio;
    return 0;
}

int process_msg3(MsgIO *msgio, IAS_Connection *ias, sgx_ra_msg1_t *msg1,
                 ra_msg4_t *msg4, config_t *config, ra_session_t *session,list<string> policy_mrsigner,list<string> policy_mrenclaves,list<sgx_prod_id_t> policy_prodids)
{
    sgx_ra_msg3_t *msg3;
    size_t blen = 0;
    size_t sz;
    int rv;
    uint32_t quote_sz;
    char *buffer = NULL;
    char *b64quote;
    sgx_mac_t vrfymac;
    sgx_quote_t *q;

    /*
     * Read our incoming message. We're using base16 encoding/hex strings
     * so we should end up with sizeof(msg)*2 bytes.
     */

    fprintf(stderr, "Waiting for msg3\n");
    // sleep(5);
    /*
     * Read message 3
     *
     * CMACsmk(M) || M
     *
     * where
     *
     * M = ga || PS_SECURITY_PROPERTY || QUOTE
     *
     */

    rv = msgio->read((void **)&msg3, &sz);
    if (rv == -1)
    {
        eprintf("system error reading msg3\n");
        return 0;
    }
    else if (rv == 0)
    {
        eprintf("protocol error reading msg3\n");
        return 0;
    }

    /*
     * The quote size will be the total msg3 size - sizeof(sgx_ra_msg3_t)
     * since msg3.quote is a flexible array member.
     *
     * Total message size is sz/2 since the income message is in base16.
     */
    quote_sz = (uint32_t)((sz / 2) - sizeof(sgx_ra_msg3_t));

    /* Make sure Ga matches msg1 */

    if (CRYPTO_memcmp(&msg3->g_a, &msg1->g_a, sizeof(sgx_ec256_public_t)))
    {
        eprintf("msg1.g_a and mgs3.g_a keys don't match\n");
        free(msg3);
        return 0;
    }

    /* Validate the MAC of M */

    cmac128(session->smk, (unsigned char *)&msg3->g_a,
            sizeof(sgx_ra_msg3_t) - sizeof(sgx_mac_t) + quote_sz,
            (unsigned char *)vrfymac);
    if (CRYPTO_memcmp(msg3->mac, vrfymac, sizeof(sgx_mac_t)))
    {
        eprintf("Failed to verify msg3 MAC\n");
        free(msg3);
        return 0;
    }

    /* Encode the report body as base64 */

    b64quote = base64_encode((char *)&msg3->quote, quote_sz);
    if (b64quote == NULL)
    {
        eprintf("Could not base64 encode the quote\n");
        free(msg3);
        return 0;
    }
    q = (sgx_quote_t *)msg3->quote;

    if (debug_server_info)
    {
        edividerWithText("Msg3 Details (from Client)");
        eprintf("msg3.mac                 = %s\n",
                hexstring(&msg3->mac, sizeof(msg3->mac)));
        eprintf("msg3.g_a.gx              = %s\n",
                hexstring(msg3->g_a.gx, sizeof(msg3->g_a.gx)));
        eprintf("msg3.g_a.gy              = %s\n",
                hexstring(&msg3->g_a.gy, sizeof(msg3->g_a.gy)));
        eprintf("msg3.ps_sec_prop         = %s\n",
                hexstring(&msg3->ps_sec_prop, sizeof(msg3->ps_sec_prop)));
        eprintf("msg3.quote.version       = %s\n",
                hexstring(&q->version, sizeof(uint16_t)));
        eprintf("msg3.quote.sign_type     = %s\n",
                hexstring(&q->sign_type, sizeof(uint16_t)));
        eprintf("msg3.quote.epid_group_id = %s\n",
                hexstring(&q->epid_group_id, sizeof(sgx_epid_group_id_t)));
        eprintf("msg3.quote.qe_svn        = %s\n",
                hexstring(&q->qe_svn, sizeof(sgx_isv_svn_t)));
        eprintf("msg3.quote.pce_svn       = %s\n",
                hexstring(&q->pce_svn, sizeof(sgx_isv_svn_t)));
        eprintf("msg3.quote.xeid          = %s\n",
                hexstring(&q->xeid, sizeof(uint32_t)));
        eprintf("msg3.quote.basename      = %s\n",
                hexstring(&q->basename, sizeof(sgx_basename_t)));
        eprintf("msg3.quote.report_body   = %s\n",
                hexstring(&q->report_body, sizeof(sgx_report_body_t)));
        eprintf("msg3.quote.signature_len = %s\n",
                hexstring(&q->signature_len, sizeof(uint32_t)));
        eprintf("msg3.quote.signature     = %s\n",
                hexstring(&q->signature, q->signature_len));

        edividerWithText("Enclave Quote (base64) ==> Send to IAS");

        eputs(b64quote);

        eprintf("\n");
        edivider();
    }

    /* Verify that the EPID group ID in the quote matches the one from msg1 */

    eprintf("+++ Validating quote's epid_group_id against msg1\n");
    if (debug_server_info)
    {
        eprintf("msg1.egid = %s\n",
                hexstring(msg1->gid, sizeof(sgx_epid_group_id_t)));
        eprintf("msg3.quote.epid_group_id = %s\n",
                hexstring(&q->epid_group_id, sizeof(sgx_epid_group_id_t)));
    }

    if (memcmp(msg1->gid, &q->epid_group_id, sizeof(sgx_epid_group_id_t)))
    {
        eprintf("EPID GID mismatch. Attestation failed.\n");
        free(b64quote);
        free(msg3);
        return 0;
    }

    if (get_attestation_report(ias, config->apiver, b64quote,
                               msg3->ps_sec_prop, msg4, config->strict_trust))
    {

        unsigned char vfy_rdata[64];
        unsigned char msg_rdata[144]; /* for Ga || Gb || VK */

        sgx_report_body_t *r = (sgx_report_body_t *)&q->report_body;

        memset(vfy_rdata, 0, 64);

        /*
         * Verify that the first 64 bytes of the report data (inside
         * the quote) are SHA256(Ga||Gb||VK) || 0x00[32]
         *
         * VK = CMACkdk( 0x01 || "VK" || 0x00 || 0x80 || 0x00 )
         *
         * where || denotes concatenation.
         */

        /* Derive VK */

        cmac128(session->kdk, (unsigned char *)("\x01VK\x00\x80\x00"),
                6, session->vk);

        /* Build our plaintext */

        memcpy(msg_rdata, session->g_a, 64);
        memcpy(&msg_rdata[64], session->g_b, 64);
        memcpy(&msg_rdata[128], session->vk, 16);

        /* SHA-256 hash */

        sha256_digest(msg_rdata, 144, vfy_rdata);
        if (debug_server_info)
        {
            edividerWithText("Enclave Report Verification");

            eprintf("VK                 = %s\n",
                    hexstring(session->vk, 16));

            eprintf("SHA256(Ga||Gb||VK) = %s\n",
                    hexstring(vfy_rdata, 32));
            eprintf("report_data[64]    = %s\n",
                    hexstring(&r->report_data, 64));
        }

        if (CRYPTO_memcmp((void *)vfy_rdata, (void *)&r->report_data,
                          64))
        {

            eprintf("Report verification failed.\n");
            free(b64quote);
            free(msg3);
            return 0;
        }

        /*
         * The service provider must validate that the enclave
         * report is from an enclave that they recognize. Namely,
         * that the MRSIGNER matches our signing key, and the MRENCLAVE
         * hash matches an enclave that we compiled.
         *
         * Other policy decisions might include examining ISV_SVN to
         * prevent outdated/deprecated software from successfully
         * attesting, and ensuring the TCB is not out of date.
         *
         * A real-world service provider might allow multiple ISV_SVN
         * values, but for this sample we only allow the enclave that
         * is compiled.
         */

#ifndef _WIN32

        if(!verify_enclave_identity(policy_mrsigner,policy_mrenclaves,policy_prodids,
        config->min_isvsvn,config->allow_debug_enclave,r))
        {

            eprintf("Invalid enclave.\n");
            msg4->status = NotTrusted;
        
        }  
#endif

        edivider();

        // The enclave report is valid so we can trust the report
        // data.

        if (debug_server_info)
        {
            edividerWithText("Enclave Report Details");

            eprintf("cpu_svn     = %s\n",
                    hexstring(&r->cpu_svn, sizeof(sgx_cpu_svn_t)));
            eprintf("misc_select = %s\n",
                    hexstring(&r->misc_select, sizeof(sgx_misc_select_t)));
            eprintf("attributes  = %s\n",
                    hexstring(&r->attributes, sizeof(sgx_attributes_t)));
            eprintf("mr_enclave  = %s\n",
                    hexstring(&r->mr_enclave, sizeof(sgx_measurement_t)));
            eprintf("mr_signer   = %s\n",
                    hexstring(&r->mr_signer, sizeof(sgx_measurement_t)));
            eprintf("isv_prod_id = %04hX\n", r->isv_prod_id);
            eprintf("isv_svn     = %04hX\n", r->isv_svn);
            eprintf("report_data = %s\n",
                    hexstring(&r->report_data, sizeof(sgx_report_data_t)));
            eprintf("[DEBUG MODE]SKIP_MRENCLAVE_ALLOWED 22112021\n");
        }

        edividerWithText("Copy/Paste Msg4 Below to Client");

        /* Serialize the members of the Msg4 structure independently */
        /* vs. the entire structure as one send_msg() */

        msgio->send_partial(&msg4->status, sizeof(msg4->status));
        msgio->send(&msg4->platformInfoBlob, sizeof(msg4->platformInfoBlob));

        edivider();

        /*
         * If the enclave is trusted, derive the MK and SK. Also get
         * SHA256 hashes of these so we can verify there's a shared
         * secret between us and the client.
         */

        if (msg4->status == Trusted)
        {
            unsigned char hashmk[32], hashsk[32];

            eprintf("+++ Deriving the MK and SK\n");
            cmac128(session->kdk, (unsigned char *)("\x01MK\x00\x80\x00"),
                    6, session->mk);
            cmac128(session->kdk, (unsigned char *)("\x01SK\x00\x80\x00"),
                    6, session->sk);

            sha256_digest(session->mk, 16, hashmk);
            sha256_digest(session->sk, 16, hashsk);

            if (debug_server_info)
            {
                eprintf("MK         = %s\n", hexstring(session->mk, 16));
                eprintf("SK         = %s\n", hexstring(session->sk, 16));

                eprintf("SHA256(MK) = %s\n", hexstring(hashmk, 32));
                eprintf("SHA256(SK) = %s\n", hexstring(hashsk, 32));
            }
        }
    }

    else
    {
        eprintf("Attestation failed\n");
        free(msg3);
        free(b64quote);
        return 0;
    }

    free(b64quote);
    free(msg3);

    return 1;
}

/*
 * Read and process message 0 and message 1. These messages are sent by
 * the client concatenated together for efficiency (msg0||msg1).
 */

int process_msg01(MsgIO *msgio, IAS_Connection *ias, sgx_ra_msg1_t *msg1,
                  sgx_ra_msg2_t *msg2, char **sigrl, config_t *config, ra_session_t *session)
{
    struct msg01_struct
    {
        uint32_t msg0_extended_epid_group_id;
        sgx_ra_msg1_t msg1;
    } * msg01;
    size_t blen = 0;
    char *buffer = NULL;
    unsigned char digest[32], r[32], s[32], gb_ga[128];
    memset(r,0,sizeof(r));
    memset(s,0,sizeof(s));
    memset(gb_ga,0,sizeof(gb_ga));
    memset(digest,0,sizeof(digest));
    EVP_PKEY *Gb;
    int rv;

    memset(msg2, 0, sizeof(sgx_ra_msg2_t));

    /*
     * Read our incoming message. We're using base16 encoding/hex strings
     * so we should end up with sizeof(msg)*2 bytes.
     */
    fprintf(stderr, "Waiting for msg0||msg1\n");

    rv = msgio->read((void **)&msg01, NULL);
    if (rv == -1)
    {
        eprintf("system error reading msg0||msg1\n");
        return 0;
    }
    else if (rv == 0)
    {
        eprintf("protocol error reading msg0||msg1\n");
        return 0;
    }

    edividerWithText("Msg0 Details (from Client)");
    eprintf("msg0.extended_epid_group_id = %u\n",
            msg01->msg0_extended_epid_group_id);
    edivider();

    /* According to the Intel SGX Developer Reference
     * "Currently, the only valid extended Intel(R) EPID group ID is zero. The
     * server should verify this value is zero. If the Intel(R) EPID group ID
     * is not zero, the server aborts remote attestation"
     */

    if (msg01->msg0_extended_epid_group_id != 0)
    {
        eprintf("msg0 Extended Epid Group ID is not zero.  Exiting.\n");
        free(msg01);
        return 0;
    }

    // Pass msg1 back to the pointer in the caller func
    memcpy(msg1, &msg01->msg1, sizeof(sgx_ra_msg1_t));
    if (debug_server_info)
    {
        edividerWithText("Msg1 Details (from Client)");
        eprintf("msg1.g_a.gx = %s\n",
                hexstring(&msg1->g_a.gx, sizeof(msg1->g_a.gx)));
        eprintf("msg1.g_a.gy = %s\n",
                hexstring(&msg1->g_a.gy, sizeof(msg1->g_a.gy)));
        eprintf("msg1.gid    = %s\n",
                hexstring(&msg1->gid, sizeof(msg1->gid)));
        edivider();
    }
    /* Generate our session key */
    eprintf("+++ generating session key Gb\n");
    Gb = key_generate();
    if (Gb == NULL)
    {
        eprintf("Could not create a session key\n");
        free(msg01);
        return 0;
    }

    /*
     * Derive the KDK from the key (Ga) in msg1 and our session key.
     * An application would normally protect the KDK in memory to
     * prevent trivial inspection.
     */

    eprintf("+++ deriving KDK\n");

    if (!derive_kdk(Gb, session->kdk, msg1->g_a, config))
    {
        eprintf("Could not derive the KDK\n");
        free(msg01);
        return 0;
    }

    if (debug_server_info)
        eprintf("+++ KDK = %s\n", hexstring(session->kdk, 16));

    /*
     * Derive the SMK from the KDK
     * SMK = AES_CMAC(KDK, 0x01 || "SMK" || 0x00 || 0x80 || 0x00)
     */

    eprintf("+++ deriving SMK\n");

    cmac128(session->kdk, (unsigned char *)("\x01SMK\x00\x80\x00"), 7,
            session->smk);

    if (debug_server_info)
        eprintf("+++ SMK = %s\n", hexstring(session->smk, 16));

    /*
     * Build message 2
     *
     * A || CMACsmk(A) || SigRL
     * (148 + 16 + SigRL_length bytes = 164 + SigRL_length bytes)
     *
     * where:
     *
     * A      = Gb || SPID || TYPE || KDF-ID || SigSP(Gb, Ga)
     *          (64 + 16 + 2 + 2 + 64 = 148 bytes)
     * Ga     = Client enclave's session key
     *          (32 bytes)
     * Gb     = Service Provider's session key
     *          (32 bytes)
     * SPID   = The Service Provider ID, issued by Intel to the vendor
     *          (16 bytes)
     * TYPE   = Quote type (0= linkable, 1= linkable)
     *          (2 bytes)
     * KDF-ID = (0x0001= CMAC entropy extraction and key derivation)
     *          (2 bytes)
     * SigSP  = ECDSA signature of (Gb.x || Gb.y || Ga.x || Ga.y) as r || s
     *          (signed with the Service Provider's private key)
     *          (64 bytes)
     *
     * CMACsmk= AES-128-CMAC(A)
     *          (16 bytes)
     *
     * || denotes concatenation
     *
     * Note that all key components (Ga.x, etc.) are in little endian
     * format, meaning the byte streams need to be reversed.
     *
     * For SigRL, send:
     *
     *  SigRL_size || SigRL_contents
     *
     * where sigRL_size is a 32-bit uint (4 bytes). This matches the
     * structure definition in sgx_ra_msg2_t
     */

    key_to_sgx_ec256(&msg2->g_b, Gb);
    sgx_spid_t spid = config->spid;
    memcpy(&msg2->spid, &spid, sizeof(sgx_spid_t)); 
    msg2->quote_type = config->quote_type;
    msg2->kdf_id = 1;

    /* Get the sigrl */

    if (!get_sigrl(ias, config->apiver, msg1->gid, sigrl,
                   &msg2->sig_rl_size))
    {

        eprintf("could not retrieve the sigrl\n");
        free(msg01);
        return 0;
    }


    memcpy(gb_ga, &msg2->g_b, 64);
    memcpy(session->g_b, &msg2->g_b, 64);

    memcpy(&gb_ga[64], &msg1->g_a, 64);
    memcpy(session->g_a, &msg1->g_a, 64);


    if (debug_server_info)
        eprintf("+++ GbGa = %s\n", hexstring(gb_ga, 128));

    EVP_PKEY* pk = config->service_private_key;

    ecdsa_sign(gb_ga, 128, pk, r, s, digest);
    reverse_bytes(&msg2->sign_gb_ga.x, r, 32);  

    reverse_bytes(&msg2->sign_gb_ga.y, s, 32);
    if (debug_server_info)
    {
        eprintf("+++ sha256(GbGa) = %s\n", hexstring(digest, 32));
        eprintf("+++ r = %s\n", hexstring(r, 32));
        eprintf("+++ s = %s\n", hexstring(s, 32));
    }

    /* The "A" component is conveniently at the start of sgx_ra_msg2_t */

    cmac128(session->smk, (unsigned char *)msg2, 148,
            (unsigned char *)&msg2->mac);

    if (debug_server_info)
    {
        edividerWithText("Msg2 Details");
        eprintf("msg2.g_b.gx      = %s\n",
                hexstring(&msg2->g_b.gx, sizeof(msg2->g_b.gx)));
        eprintf("msg2.g_b.gy      = %s\n",
                hexstring(&msg2->g_b.gy, sizeof(msg2->g_b.gy)));
        eprintf("msg2.spid        = %s\n",
                hexstring(&msg2->spid, sizeof(msg2->spid)));
        eprintf("msg2.quote_type  = %s\n",
                hexstring(&msg2->quote_type, sizeof(msg2->quote_type)));
        eprintf("msg2.kdf_id      = %s\n",
                hexstring(&msg2->kdf_id, sizeof(msg2->kdf_id)));
        eprintf("msg2.sign_ga_gb  = %s\n",
                hexstring(&msg2->sign_gb_ga, sizeof(msg2->sign_gb_ga)));
        eprintf("msg2.mac         = %s\n",
                hexstring(&msg2->mac, sizeof(msg2->mac)));
        eprintf("msg2.sig_rl_size = %s\n",
                hexstring(&msg2->sig_rl_size, sizeof(msg2->sig_rl_size)));
        edivider();
    }
    free(msg01);

    return 1;
}

int derive_kdk(EVP_PKEY *Gb, unsigned char kdk[16], sgx_ec256_public_t g_a,
               config_t *config)
{
    unsigned char *Gab_x;
    size_t slen;
    EVP_PKEY *Ga;
    unsigned char cmackey[16];

    memset(cmackey, 0, 16);

    /*
     * Compute the shared secret using the peer's public key and a generated
     * public/private key.
     */

    Ga = key_from_sgx_ec256(&g_a);
    if (Ga == NULL)
    {
        crypto_perror("key_from_sgx_ec256");
        return 0;
    }

    /* The shared secret in a DH exchange is the x-coordinate of Gab */
    Gab_x = key_shared_secret(Gb, Ga, &slen);
    if (Gab_x == NULL)
    {
        crypto_perror("key_shared_secret");
        return 0;
    }

    /* We need it in little endian order, so reverse the bytes. */
    /* We'll do this in-place. */

    reverse_bytes(Gab_x, Gab_x, slen);

    if (debug_server_info)
        eprintf("+++ Shared secret reversed     = %s\n", hexstring(Gab_x, slen));

    /* Now hash that to get our KDK (Key Definition Key) */

    /*
     * KDK = AES_CMAC(0x00000000000000000000000000000000, secret)
     */

    cmac128(cmackey, Gab_x, slen, kdk);

    return 1;
}

int get_sigrl(IAS_Connection *ias, int version, sgx_epid_group_id_t gid,
              char **sig_rl, uint32_t *sig_rl_size)
{
    IAS_Request *req = NULL;
    int oops = 1;
    string sigrlstr;

    try
    {
        oops = 0;
        req = new IAS_Request(ias, (uint16_t)version);
    }
    catch (...)
    {
        oops = 1;
    }

    if (oops)
    {
        eprintf("Exception while creating IAS request object\n");
        delete req;
        return 0;
    }

    ias_error_t ret = IAS_OK;

    while (1)
    {

        ret = req->sigrl(*(uint32_t *)gid, sigrlstr);
        if (debug_server_info == 0)
        {
            eprintf("+++ RET = %zu\n, ret");
            eprintf("+++ SubscriptionKeyID = %d\n", (int)ias->getSubscriptionKeyID());
        }

        if (ret == IAS_UNAUTHORIZED && (ias->getSubscriptionKeyID() == IAS_Connection::SubscriptionKeyID::Primary))
        {
            eprintf("+++ IAS Primary Subscription Key failed with IAS_UNAUTHORIZED\n");
            eprintf("+++ Retrying with IAS Secondary Subscription Key\n");

            // Retry with Secondary Subscription Key
            ias->SetSubscriptionKeyID(IAS_Connection::SubscriptionKeyID::Secondary);
            continue;
        }
        else if (ret != IAS_OK)
        {

            delete req;
            return 0;
        }

        break;
    }
    size_t size;
    *sig_rl = (char *)base64_decode(sigrlstr.c_str(), &size);
    if (*sig_rl == NULL)
    {
        eprintf("Could not decode SigRL\n");
        delete req;
        return 0;
    }

    *sig_rl_size = (uint32_t)size;
    delete req;

    return 1;
}

int get_attestation_report(IAS_Connection *ias, int version,
                           const char *b64quote, sgx_ps_sec_prop_desc_t secprop, ra_msg4_t *msg4,
                           int strict_trust)
{
    IAS_Request *req = NULL;
    map<string, string> payload;
    vector<string> messages;
    ias_error_t status;
    string content;

    try
    {
        req = new IAS_Request(ias, (uint16_t)version);
    }
    catch (...)
    {
        eprintf("Exception while creating IAS request object\n");
        if (req != NULL)
            delete req;
        return 0;
    }

    payload.insert(make_pair("isvEnclaveQuote", b64quote));

    status = req->report(payload, content, messages);
    if (status == IAS_OK)
    {
        JSON reportObj = JSON::Load(content);
        if (debug_server_info)
        {
            edividerWithText("Report Body");
            eprintf("%s\n", content.c_str());
            edivider();
            if (messages.size())
            {
                edividerWithText("IAS Advisories");
                for (vector<string>::const_iterator i = messages.begin();
                     i != messages.end(); ++i)
                {

                    eprintf("%s\n", i->c_str());
                }
                edivider();
            }
            if (debug_server_info)
            {
                edividerWithText("IAS Report - JSON - Required Fields");
                if (version >= 3)
                {
                    eprintf("version=           %d\n", reportObj["version"].ToInt());
                }
                eprintf("id:                   = %s\n",
                        reportObj["id"].ToString().c_str());
                eprintf("timestamp             = %s\n",
                        reportObj["timestamp"].ToString().c_str());

                eprintf("isvEnclaveQuoteStatus = %s\n",
                        reportObj["isvEnclaveQuoteStatus"].ToString().c_str());
                eprintf("isvEnclaveQuoteBody   = %s\n",
                        reportObj["isvEnclaveQuoteBody"].ToString().c_str());

                edividerWithText("IAS Report - JSON - Optional Fields");

                eprintf("platformInfoBlob  = %s\n",
                        reportObj["platformInfoBlob"].ToString().c_str());
                eprintf("revocationReason  = %s\n",
                        reportObj["revocationReason"].ToString().c_str());
                eprintf("pseManifestStatus = %s\n",
                        reportObj["pseManifestStatus"].ToString().c_str());
                eprintf("pseManifestHash   = %s\n",
                        reportObj["pseManifestHash"].ToString().c_str());
                eprintf("nonce             = %s\n",
                        reportObj["nonce"].ToString().c_str());
                eprintf("epidPseudonym     = %s\n",
                        reportObj["epidPseudonym"].ToString().c_str());
                if (version >= 4)
                {
                    int i;

                    eprintf("advisoryURL       = %s\n",
                            reportObj["advisoryURL"].ToString().c_str());
                    eprintf("advisoryIDs       = ");
                    /* This is a JSON array */
                    for (i = 0; i < reportObj["advisoryIDs"].length(); ++i)
                    {
                        eprintf("%s%s", (i) ? "," : "", reportObj["advisoryIDs"][i].ToString().c_str());
                    }
                    eprintf("\n");
                }
                edivider();

                /*
                 * If the report returned a version number (API v3 and above), make
                 * sure it matches the API version we used to fetch the report.
                 *
                 * For API v3 and up, this field MUST be in the report.
                 */

                if (reportObj.hasKey("version"))
                {
                    unsigned int rversion = (unsigned int)reportObj["version"].ToInt();

                    eprintf("+++ Verifying report version against API version\n");
                    if (version != rversion)
                    {
                        eprintf("Report version %u does not match API version %u\n",
                                rversion, version);
                        delete req;
                        return 0;
                    }
                }
                else if (version >= 3)
                {
                    eprintf("attestation report version required for API version >= 3\n");
                    delete req;
                    return 0;
                }
            }
        }

        /*
         * This sample's attestion policy is based on isvEnclaveQuoteStatus:
         *
         *   1) if "OK" then return "Trusted"
         *
         *   2) if "CONFIGURATION_NEEDED", "SW_HARDENING_NEEDED", or
         *      "CONFIGURATION_AND_SW_HARDENING_NEEDED", then return
                "NotTrusted_ItsComplicated" when in --strict-trust-mode
                 and "Trusted_ItsComplicated" otherwise
         *
         *   3) return "NotTrusted" for all other responses
         *
         * In case #2, this is ultimatly a policy decision. Do you want to
         * trust a client that is running with a configuration that weakens
         * its security posture? Even if you ultimately choose to trust the
         * client, the "Trusted_ItsComplicated" response is intended to
         * tell the client "I'll trust you (for now), but inform the user
         * that I may not trust them in the future unless they take some
         * action". A real service would provide some guidance to the
         * end user based on the advisory URLs and advisory IDs.
         */

        /*
         * Simply check to see if status is OK, else enclave considered
         * not trusted
         */

        memset(msg4, 0, sizeof(ra_msg4_t));

        edividerWithText("ISV Enclave Trust Status");

        if (!(reportObj["isvEnclaveQuoteStatus"].ToString().compare("OK")))
        {
            msg4->status = Trusted;

            eprintf("Enclave TRUSTED\n");
        }
        else if (!(reportObj["isvEnclaveQuoteStatus"].ToString().compare("CONFIGURATION_NEEDED")))
        {
            if (strict_trust)
            {
                msg4->status = NotTrusted_ItsComplicated;

                eprintf("Enclave NOT TRUSTED and COMPLICATED - Reason: %s\n",
                        reportObj["isvEnclaveQuoteStatus"].ToString().c_str());
            }
            else
            {

                eprintf("Enclave TRUSTED and COMPLICATED - Reason: %s\n",
                        reportObj["isvEnclaveQuoteStatus"].ToString().c_str());
                msg4->status = Trusted_ItsComplicated;
            }
        }
        else if (!(reportObj["isvEnclaveQuoteStatus"].ToString().compare("GROUP_OUT_OF_DATE")))
        {
            msg4->status = NotTrusted_ItsComplicated;

            eprintf("Enclave NOT TRUSTED and COMPLICATED - Reason: %s\n",
                    reportObj["isvEnclaveQuoteStatus"].ToString().c_str());
        }
        else
        {
            msg4->status = NotTrusted;

            eprintf("Enclave NOT TRUSTED - Reason: %s\n",
                    reportObj["isvEnclaveQuoteStatus"].ToString().c_str());
        }

        /* Check to see if a platformInfoBlob was sent back as part of the
         * response */

        if (!reportObj["platformInfoBlob"].IsNull())
        {

            eprintf("A Platform Info Blob (PIB) was provided by the IAS\n");

            /* The platformInfoBlob has two parts, a TVL Header (4 bytes),
             * and TLV Payload (variable) */

            string pibBuff = reportObj["platformInfoBlob"].ToString();

            /* remove the TLV Header (8 base16 chars, ie. 4 bytes) from
             * the PIB Buff. */

            pibBuff.erase(pibBuff.begin(), pibBuff.begin() + (4 * 2));

            int ret = from_hexstring((unsigned char *)&msg4->platformInfoBlob,
                                     pibBuff.c_str(), pibBuff.length() / 2);
        }
        else
        {

            eprintf("A Platform Info Blob (PIB) was NOT provided by the IAS\n");
        }

        delete req;
        return 1;
    }

    eprintf("attestation query returned %lu: \n", status);

    switch (status)
    {
    case IAS_QUERY_FAILED:
        eprintf("Could not query IAS\n");
        break;
    case IAS_BADREQUEST:
        eprintf("Invalid payload\n");
        break;
    case IAS_UNAUTHORIZED:
        eprintf("Failed to authenticate or authorize request\n");
        break;
    case IAS_SERVER_ERR:
        eprintf("An internal error occurred on the IAS server\n");
        break;
    case IAS_UNAVAILABLE:
        eprintf("Service is currently not able to process the request. Try again later.\n");
        break;
    case IAS_INTERNAL_ERROR:
        eprintf("An internal error occurred while processing the IAS response\n");
        break;
    case IAS_BAD_CERTIFICATE:
        eprintf("The signing certificate could not be validated\n");
        break;
    case IAS_BAD_SIGNATURE:
        eprintf("The report signature could not be validated\n");
        break;
    default:
        if (status >= 100 && status < 600)
        {
            eprintf("Unexpected HTTP response code\n");
        }
        else
        {
            eprintf("An unknown error occurred.\n");
        }
    }

    delete req;

    return 0;
}

#ifndef _WIN32

/* We don't care which signal it is since we're shutting down regardless */

void cleanup_and_exit(int signo)
{
    /* Signal-safe, and we don't care if it fails or is a partial write. */

    ssize_t bytes = write(STDERR_FILENO, "\nterminating\n", 13);

    /*
     * This destructor consists of signal-safe system calls (close,
     * shutdown).
     */

    delete msgio;

    exit(1);
}
#endif

#define NNL << endl \
            << endl \
            <<
#define NL << endl \
           <<

void usage()
{
    cerr << "Parameters incorrept,aborting" << endl;
    cleanup_and_exit(2);
}
