import regex
import ctagsInterface

class CodeSplittingInterface:

	def is_number(n: str):
		try:
			float(n)  # Type-casting the string to `float`. If string is not a valid `float`,it'll raise `ValueError` exception
		except ValueError:
			return False
		return True

	def checkTypeParams(arguments: list):
		"""Receives the type of the arguments of the function and checks that their types are basic """
		basic_types = ["int", "double", "float", "char", "unsigned_int"]
		for a in arguments:
			if a not in basic_types:
				return -1
		return 0

	def checkNoGlobalVars(codeFunction: str, params: str,filepath: str):
		"""Receives the implementation code of the function and check that no global vars are used inside"""

		global_vars =  ctagsInterface.getGlobalVars(filepath)  # Call ctags and get a List of the names of the global variables of the source code
		if not global_vars:
			return 0

		vars_used = set(regex.findall(r"([\t]*[\w\d]+[\t]*)", codeFunction, regex.MULTILINE))
		params_used = params.split(",")
		for v in global_vars:
			if v in vars_used and v not in params_used and not CodeSplittingInterface.is_number(v):  #to distinguish values from variables (ex: 1 is not var name "1")
				return -1

		return 0

	def checkVarsCopiedByValue(arguments: list):
		"""Receives the parameters of the functions and checks that they are copied by value and not by reference"""
		for a in arguments:
			typeVar, name = a.split(" ")[0], a.split(" ")[1]
			# check that the var is not a pointer
			if typeVar.endswith("*") or name.startswith("*"):
				return -1

		return 0

	def returnTypeParam(typeParam: str):
		"""Receives a string (ex.int) and returns a character symbol for the type"""
		if typeParam == "int":
			return "i", "value"
		elif typeParam == "char":
			return "c", "buf"
		elif typeParam == "double":
			return "d", "value_d"
		elif typeParam == "float":
			return "f", "value_f"
		elif typeParam == "char*" or typeParam == "char *":
			return "c*", "buf"
		elif typeParam == "unsigned_int":
			return "u", "value_u"

	def returnFunctionArgsCRF(arguments: str, num_arguments: int):
		"""Return the parameters of the function called by the server """
		args = "("
		vect = arguments.replace("(", "").replace(")", "").split(",")
		for i in range(num_arguments):
			type_arg = vect[i].lstrip().split(" ")[0]
			if type_arg == "int":
				args += f"p[{i}].value"
			elif type_arg == "float":
				args += f"p[{i}].value_f"
			elif type_arg == "char" or type_arg == "char*" or vect[i].lstrip().split(" ")[1].lstrip().startswith("*"):
				args += f"p[{i}].buf"
			elif type_arg == "double":
				args += f"p[{i}].value_d"
			elif type_arg == f"unsigned" and vect[i].lstrip().split(" ")[1] == "int":
				args += f"p[{i}].value_u"

			if i != num_arguments - 1:
				args += ","
		args += ")"
		return args

	def returnTypeResultFunction(type_return: str):
		""" It returns the right parameter for the type of function called(ex:int->res_call;char->res_char;float->res_float"""
		if type_return == "int":
			return "res_call"
		elif type_return == "float":
			return "res_float"
		elif type_return == "char" or (type_return == "char" and '*' in type_return):
			return "res_char"
		elif type_return == "double":
			return "res_double"
		elif type_return == "unsigned_int":
			return "res_ui"

	def returnFormatSpecifiers(type_return: str):
		""" Returns the format specifiers for the type of function called (ex: int->%d,float->%f)"""
		if type_return == "int":
			return "%d"
		elif type_return == "float" or type_return == "double" :
			return "%f"
		elif type_return == "char" :
			return "%c"
		elif type_return == "char" and '*' in type_return:
			return "%s"
		elif type_return == "unsigned int":
			return "%i"

	__crfImplementationCode__ = r"""char message[1024] = "ok";
	char buffer[1024],response[11];
	int clientSocket,n1,n2 = 0;
	struct sockaddr_in serverAddr;
	socklen_t addr_size;
	bzero(buffer,1024);
	//insertion of attest_enclave for RA
    sgx_enclave_id_t eid1 = create_enclave_1();
	attest_enclave(eid1);
	// Create the socket.
	clientSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (clientSocket == -1)
	{
		printf("socket creation failed...\n");
		exit(0);
	}
	// Configure settings of the server address
	//  Address family is Internet
	serverAddr.sin_family = AF_INET;

	// Set port number, using htons function
	serverAddr.sin_port = htons(@REPLACE_DEFAULT_PORT@);

	// Set IP address to localhost
	serverAddr.sin_addr.s_addr = inet_addr("127.0.0.1");
	// Connect the socket to the server using the address
	addr_size = sizeof serverAddr;
	connect(clientSocket, (struct sockaddr *)&serverAddr, addr_size);
	sprintf(buffer, "%d", f->function_id); /* since in unix, itoa is not defined in standard */

	n1 = send(clientSocket, buffer, sizeof(f->function_id), 0);
	if (n1 < 0)
	{
		printf("Send failed\n");
		abort();
	}
	// Sending PARAMETERS
	for (int i = 0; i < f->num_arguments; i++)
	{
		if(f->arguments[i].type == 'i')
		sprintf(buffer, "Type:%c Value:%d", f->arguments[i].type, f->arguments[i].value);
		else if(f->arguments[i].type == 'f')
		sprintf(buffer, "Type:%c Value:%f", f->arguments[i].type, f->arguments[i].value_f);
		else if(f->arguments[i].type == 'c')
		sprintf(buffer, "Type:%c Value:%s", f->arguments[i].type, f->arguments[i].buf);
		else if(f->arguments[i].type == 'd')
		sprintf(buffer, "Type:%c Value:%f", f->arguments[i].type, f->arguments[i].value_d);
		else if(f->arguments[i].type == 'u')
		sprintf(buffer, "Type:%c Value:%i", f->arguments[i].type, f->arguments[i].value_u);
		n2 += send(clientSocket, buffer,1024, 0); // sizeof(buffer) = 1024 B
		printf("DEBUG:Bytes send to server:%d\n",n2); 
		if ( n2 < 0)
		{
			printf("Send parameter %d failed\n", i + 1);
			abort();
		}
	}
	//Send message "ok" signals that the params are finished and we are waiting for the result
	if (send(clientSocket, message, sizeof(message), 0) < 0)
	{
		printf("Send failed\n");
		abort();
	}
	//  Read the message from the server into the buffer
	int n = recv(clientSocket, buffer, 1024, 0);
	if (n < 0)
	{
		printf("Receive failed\n");
	}
	char response[11];

	strncpy(response,buffer,n);
	response[sizeof(buffer)] = '\0';
	printf("DEBUG:read bytes:%d; buffer = %s\n", n, response);
	// Print the received message
	if(f->return_type=='i'){
		f->return_value.value = atoi(buffer);
	}else if(f->return_type=='c'){
		strcpy(f->return_value.buf,response);
	}else if(f->return_type=='f'){
		f->return_value.value_f = atof(response);
	}else if(f->return_type=='d'){
		f->return_value.value_d = atof(response); //not robust, can be substituted with strtod
	}else if(f->return_type=='u'){
		f->return_value.value_u = atof(response); 
	}
	close(clientSocket);
	return 0; #0 stands for no error
	"""

	__crfEntireCode__ = r"""int call_remote_function(function_t *f)
{
	char message[1024] = "ok";
	char buffer[1024],response[11];
	int clientSocket,n1,n2 = 0;
	struct sockaddr_in serverAddr;
	socklen_t addr_size;
	bzero(buffer,1024);
	
	//insertion of attest_enclave for RA
	sgx_enclave_id_t eid1 = create_enclave_1();
	attest_enclave(eid1);
	// Create the socket.
	clientSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (clientSocket == -1)
	{
		printf("socket creation failed...\n");
		exit(0);
	}
	// Configure settings of the server address
	//  Address family is Internet
	serverAddr.sin_family = AF_INET;

	// Set port number, using htons function
	serverAddr.sin_port = htons(@REPLACE_DEFAULT_PORT@);

	// Set IP address to localhost
	serverAddr.sin_addr.s_addr = inet_addr("127.0.0.1");
	// Connect the socket to the server using the address
	addr_size = sizeof serverAddr;
	connect(clientSocket, (struct sockaddr *)&serverAddr, addr_size);
	sprintf(buffer, "%d", f->function_id); /* since in unix, itoa is not defined in standard */

	n1 = send(clientSocket, buffer, sizeof(f->function_id), 0);
	if (n1 < 0)
	{
		printf("Send failed\n");
		abort();
	}
	// Sending PARAMETERS
	for (int i = 0; i < f->num_arguments; i++)
	{
		if(f->arguments[i].type == 'i')
		sprintf(buffer, "Type:%c Value:%d", f->arguments[i].type, f->arguments[i].value);
		else if(f->arguments[i].type == 'f')
		sprintf(buffer, "Type:%c Value:%f", f->arguments[i].type, f->arguments[i].value_f);
		else if(f->arguments[i].type == 'c')
		sprintf(buffer, "Type:%c Value:%s", f->arguments[i].type, f->arguments[i].buf);
		else if(f->arguments[i].type == 'd')
		sprintf(buffer, "Type:%c Value:%f", f->arguments[i].type, f->arguments[i].value_d);
		else if(f->arguments[i].type == 'u')
		sprintf(buffer, "Type:%c Value:%i", f->arguments[i].type, f->arguments[i].value_u);
		n2 += send(clientSocket, buffer,1024, 0); // sizeof(buffer) = 1024 B
		printf("DEBUG:Bytes send to server:%d\n",n2); //DEBUG, remove at end
		if ( n2 < 0)
		{
			printf("Send parameter %d failed\n", i + 1);
			return -2;
		}
	}
	//Send message "ok" signals that the params are finished and we are waiting for the result
	if (send(clientSocket, message, sizeof(message), 0) < 0)
	{
		printf("Send failed\n");
		return -2;
	}
	//  Read the message from the server into the buffer
	int n = recv(clientSocket, buffer, 1024, 0);
	if (n < 0)
	{
		printf("Receive failed\n");
		return -2;
	}

	strncpy(response,buffer,n);
	response[sizeof(buffer)] = '\0';
	printf("DEBUG:read bytes:%d; buffer = %s\n", n, response);
	// Print the received message
	if(f->return_type=='i'){
		f->return_value.value = atoi(buffer);
	}else if(f->return_type=='c'){
		strcpy(f->return_value.buf,response);
	}else if(f->return_type=='f'){
		f->return_value.value_f = atof(response);
	}else if(f->return_type=='d'){
		f->return_value.value_d = atof(response); //not robust, can be substituted with strtod
	}else if(f->return_type=='u'){
		long tmp = atol(response);
		f->return_value.value_u = (unsigned int) tmp; //can be substituted with strtoul
	}
	close(clientSocket);
	return 0; //0 stands for no error
}
"""
	__serverCSEntireCode__ = r"""#include <stdio.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <float.h>
#include "server_material/server_interface.h"

#define debug_messages_on 0
#define MAX 1024 
#define MAX_PARAMETERS {MAX_PARAMETERS_STUB}
#define PORT @REPLACE_DEFAULT_PORT@ 
#define SA struct sockaddr
#define MAX_CONNECTIONS 5

typedef struct{
	char type; // i = int f = float d = double c = char 
	int value;
	float value_f;
	char* buf;
	double value_d;
	unsigned int value_u;
} param_t;

typedef struct pthread_arg_t
{
    int new_socket_fd;
    struct sockaddr_in client_address;
    /* TODO: Put arguments passed to threads here. See lines 116 and 139. */
} pthread_arg_t;

[CODE_FUNCTIONS]

/* Signal handler to handle SIGTERM and SIGINT signals. */
void signal_handler(int signal_number);
/* Thread routine to start the server that will contact the IAS */
void *pthread_server(void *arg);
/* Thread routine to serve connection to client. */
void *pthread_routine(void *arg);

void *pthread_server(void *arg)
{
    int result_client_attestation = startServerIAS();
    if(debug_messages_on)
    printf("DEBUG:The returned address is equal to: %d\n",result_client_attestation);
    return NULL;
}

void *pthread_routine(void *arg)
{
    pthread_arg_t *pthread_arg = (pthread_arg_t *)arg;
    int new_socket_fd = pthread_arg->new_socket_fd;
    // struct sockaddr_in client_address = pthread_arg->client_address;
    /* TODO: Get arguments passed to threads here. See lines 22 and 116. */

    free(arg);

    /* TODO: Put client interaction code here. For example, use
     * write(new_socket_fd,,) and read(new_socket_fd,,) to send and receive
     * messages with the client.
     */
    char buff[MAX];

    int n = 0, res_call = 0;
    float res_float = 0;
    char *re_char;
    double res_double = 0;
    // Accept the data packet from client and verification

    if(debug_messages_on)
    printf("DEBUG:Start real server activities\n");
    for (;;)
    {
        // read 4 bytes for the function id (int = 32 bit = 4 Bytes)
        do
        {
            n = read(new_socket_fd, buff, 4);
        } while (n <= 0);

        // print buffer which contains the client contents
        if(debug_messages_on)
        printf("DEBUG:From client: the id of the function is %s\n", buff);
        int id = atoi(buff);
        int i = 0;
        param_t p[MAX_PARAMETERS];
        memset(p, 0, sizeof(p));
        while (1)
        {
            n = read(new_socket_fd, buff, 1024); // Type:X value:X
            if(debug_messages_on)
            printf("DEBUG:Received %s\n", buff); //
            if (strncmp(buff, "ok", 2) == 0)
                break;
            if (buff[5] == 'i')
            {
                char *value;
                /* We split the string Type:X value:X in
                 * Type:X Value:Y -> Type:X;Value:Y
                 * Type:X;Value:Y -> Value:Y
                 * Value:Y -> Value;Y
                 * Value;Y -> Y
                 * And Y is the value (in char format, to be converted)

                */

                value = strtok(buff, " ");
                value = strtok(NULL, " ");
                value = strtok(value, ":");
                value = strtok(NULL, ":");
                p[i].type = 'i';
                p[i].value = atoi(value);
            }
            else if (buff[5] == 'f')
            {
                char *value;
                /* We split the string Type:X value:X in
                 * Type:X Value:Y -> Type:X;Value:Y
                 * Type:X;Value:Y -> Value:Y
                 * Value:Y -> Value;Y
                 * Value;Y -> Y
                 * And Y is the value (in char format, to be converted)

                */

                value = strtok(buff, " ");
                value = strtok(NULL, " ");
                value = strtok(value, ":");
                value = strtok(NULL, ":");
                p[i].type = 'f';
                p[i].value_f = atof(value);
            }
            else if (buff[5] == 'c')
            {
                char *value;
                /* We split the string Type:X value:X in
                 * Type:X Value:Y -> Type:X;Value:Y
          0       * Type:X;Value:Y -> Value:Y
                 * Value:Y -> Value;Y
                 * Value;Y -> Y
                 * And Y is the value (in char format, to be converted)

                */

                value = strtok(buff, " ");
                value = strtok(NULL, " ");
                value = strtok(value, ":");
                value = strtok(NULL, ":");
                p[i].type = 'c';
                strcpy(p[i].buf, value);
            } else if (buff[5] == 'u')
            {
                char *value;
                /* We split the string Type:X value:X in
                 * Type:X Value:Y -> Type:X;Value:Y
                 * Type:X;Value:Y -> Value:Y
                 * Value:Y -> Value;Y
                 * Value;Y -> Y
                 * And Y is the value (in char format, to be converted)

                */

                value = strtok(buff, " ");
                value = strtok(NULL, " ");
                value = strtok(value, ":");
                value = strtok(NULL, ":");
                p[i].type = 'u';
                long tmp1 = atol(value);
                p[i].value_u = (unsigned int) tmp1;
            }
            i++;
        }
        switch (id)
        {

        [SWITCH_CASE_FUNCTIONS]

        default:
            res_call = -1;
            printf("Function not recognized.\n");
            break;
        }

        bzero(buff, MAX);
        break;
    }

    close(new_socket_fd);
    return NULL;
}

int main()
{
    int port, sockfd, new_socket_fd;
    struct sockaddr_in address;
    pthread_attr_t pthread_attr;
    pthread_arg_t *pthread_arg;
    pthread_t pthread;
    socklen_t client_address_len;

    /* Initialise IPv4 address. */
    memset(&address, 0, sizeof address);
    address.sin_family = AF_INET;
    address.sin_port = htons(PORT);
    address.sin_addr.s_addr = INADDR_ANY;

    /* Create TCP socket. */
    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
    {
        perror("socket");
        exit(1);
    }

    /* Bind address to socket. */
    if (bind(sockfd, (struct sockaddr *)&address, sizeof address) == -1)
    {
        perror("bind");
        exit(1);
    }

    /* Listen on socket. */
    if (listen(sockfd, MAX_CONNECTIONS) == -1)
    {
        perror("listen");
        exit(1);
    }

    /* Assign signal handlers to signals. */
    if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
    {
        perror("signal");
        exit(1);
    }
    if (signal(SIGTERM, signal_handler) == SIG_ERR)
    {
        perror("signal");
        exit(1);
    }
    if (signal(SIGINT, signal_handler) == SIG_ERR)
    {
        perror("signal");
        exit(1);
    }

    /* Initialise pthread attribute to create detached threads. */
    if (pthread_attr_init(&pthread_attr) != 0)
    {
        perror("pthread_attr_init");
        exit(1);
    }
    if (pthread_attr_setdetachstate(&pthread_attr, PTHREAD_CREATE_DETACHED) != 0)
    {
        perror("pthread_attr_setdetachstate");
        exit(1);
    }



    while (1)
    {
		if (pthread_create(&pthread, &pthread_attr, pthread_server, (void *)pthread_arg) != 0)
        {
            perror("pthread_create");
            free(pthread_arg);
        }
        /* Accept connection to client. */
        struct sockaddr *client_address;
        client_address_len = sizeof(struct sockaddr_in);
        new_socket_fd = accept(sockfd, (struct sockaddr *)&client_address, &client_address_len);
        if (new_socket_fd == -1)
        {
            perror("accept");
            free(pthread_arg);
            continue;
        }

        printf("Connection accepted and thread initialization \n");
        /* Create pthread argument for each connection to client. */
        /* TODO: malloc'ing before accepting a connection causes only one small
         * memory when the program exits. It can be safely ignored.
         */
        pthread_arg = (pthread_arg_t *)malloc(sizeof *pthread_arg);
        if (!pthread_arg)
        {
            perror("malloc");
            continue;
        }
        if(debug_messages_on)
        printf("DEBUG:pthread_arg allocated\n");
        /* Initialise pthread argument. */
        pthread_arg->new_socket_fd = new_socket_fd;
        //(struct sockaddr_in *)pthread_arg->client_address = &client_address;
       /* TODO: Initialise arguments passed to threads here. See lines 22 and
         * 139.
         */

        /* Create thread to serve connection to client. */
        if (pthread_create(&pthread, &pthread_attr, pthread_routine, (void *)pthread_arg) != 0)
        {
            perror("pthread_create");
            free(pthread_arg);
            continue;
        }
    }

    close(sockfd);
    /* close(sockfd);
     * TODO: If you really want to close the socket, you would do it in
     * signal_handler(), meaning sockfd would need to be a global variable.
     */
    return 0;
}


void signal_handler(int signal_number)
{
    /* TODO: Put exit cleanup code here. */
    exit(0);
}"""

	__server_compilation_makefile_code__ = r"""################################################
#############    SERVER_CS     #################
################################################


server_material/common.o: server_material/common.cpp
	@$(CXX) -g $(App_Include_Paths) $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN => $@"

server_material/iasrequest.o: server_material/iasrequest.cpp
	@$(CXX) -g $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN => $@"

	
server_material/agent_curl.o: server_material/agent_curl.cpp
	@$(CXX) -g $(App_Include_Paths) -lcurl -c $< -o $@
	@echo "GEN => $@"

server_material/byteorder.o: server_material/byteorder.c
	@$(CC) -g -c $< -o $@
	@echo "GEN => $@"
	
server_material/crypto.o: server_material/crypto.c
	@$(CC) -g $(App_Include_Paths) -c $< -o $@
	@echo "GEN => $@"
	
server_material/enclave_verify.o: server_material/enclave_verify.cpp
	@$(CXX) -g $(App_Include_Paths) $(SGX_COMMON_CXXFLAGS) -c $< -o $@
	@echo "GEN => $@"
	
server_material/hexutil.o: server_material/hexutil.c
	@$(CC) -g -c $< -o $@
	@echo "GEN => $@"

server_material/agent_wget.o: server_material/agent_wget.cpp
	@$(CXX) -g $(App_Include_Paths) -c $< -o $@
	@echo "GEN => $@"
	

server_material/fileio.o: server_material/fileio.c
	@$(CC) -g $(App_Include_Paths) -c $< -o $@
	@echo "GEN => $@"
	
server_material/base64.o: server_material/base64.c
	@$(CC) -g $(App_Include_Paths) -c $< -o $@
	@echo "GEN => $@" 

server_material/msgio.o : server_material/msgio.cpp
	@$(CXX) -g $(App_Include_Paths) $(SGX_COMMON_CXXFLAGS) $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN  => $@"

server_material/server_interface.cpp: server_material/server_interface.h 

server_material/server_interface.o: server_material/server_interface.cpp 
	@echo "######## CREATING $@ #########"
	@$(CXX) -g -std=c++17 $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN => $@"

server_cs.o:server_cs.c server_material/server_interface.o
	@$(CXX) -g -std=c++17 -Wno-unused-parameter $(App_Cpp_Flags) -c $< -o $@
	@echo "CXX <= $<"
	
 server_cs: server_material/server_interface.o server_material/agent_wget.o server_material/iasrequest.o server_material/enclave_verify.o server_material/byteorder.o server_material/common.o server_material/crypto.o server_material/hexutil.o server_material/fileio.o server_material/base64.o server_material/msgio.o server_material/agent_curl.o server_cs.o
	@$(CXX) -g $^ -o $@ $(App_Link_Flags) -lcurl -lcrypto
	@echo "LINK => $@" 
	
######## Enclave Objects ########
	"""
