import logging


class Constants(object):

    def writeEnclaveManager(enclave_iterator:int,remote_attestation,enclaveToAttest):
        """ Try to create a new file, override the file if already exists """
        filePath="enclave_manager_"+str(enclave_iterator)
        with open(filePath+".h", "w") as file:
            logging.info("Writing file: " + filePath)
            text = Constants.__enclave_manager_h_includes__.replace(r"""ENCLAVE_NAME "libenclave.signed.so""""",f"""ENCLAVE{str(enclave_iterator)}_NAME "libenclave{str(enclave_iterator)}.signed.so""")

            text = text.replace(r"""#ifndef _ENCLAVE_MANAGER_
#define _ENCLAVE_MANAGER_""",f"""#ifndef _ENCLAVE_MANAGER_{str(enclave_iterator)}_
#define _ENCLAVE_MANAGER_{str(enclave_iterator)}_""")
            text = text.replace(r"""#include "enclave_u.h""""",f"""#include "untrusted{str(enclave_iterator)}/enclave{str(enclave_iterator)}_u.h""""")
            file.write(text)
            # if remote_attestation == 1 and (enclaveToAttest.__len__()>1 and str(enclave_iterator) in enclaveToAttest[0]):
            #     file.write(r"""#include "../remote_attestation/client_starter.h" """)
            text = Constants.__enclave_manager_h_declarations__.replace(r"""@prototype_attest_enclave@;""",f"""int create_enclave_{str(enclave_iterator)}(void);""")
            text = text.replace(r"""my_enclave_id =""",f"""my_enclave{str(enclave_iterator)}_id ={str(enclave_iterator-1)};""")
            text = text.replace(r"""destroy_enclave(void)""", f"""destroy_enclave_{str(enclave_iterator)}(void)""")
            file.write(text)
            # if remote_attestation == 1 and (enclaveToAttest.__len__()>1 and str(enclave_iterator) in enclaveToAttest[0]):
            #     file.write(f"""\nint attest_enclave(sgx_enclave_id_t eid);""")
            file.write("\n#endif /* !_ENCLAVE_MANAGER_ */")
            file.close()
        with open(filePath+".c", "w") as file:
            # Can be changed to an invocation of a different method
            logging.info("Writing file: " + filePath+".c")
            text = Constants.__enclave_manager_c_code__.replace("enclave_manager.h",f"""enclave_manager_{str(enclave_iterator)}.h""")
            text = text.replace(r"""@replace_enclave_id@""",f"{str(enclave_iterator-1)}")
            text = text.replace(r"""@prototype_attest_enclave@""",f"""int create_enclave_{str(enclave_iterator)}(void)""")
            text = text.replace("my_enclave_id",f"""my_enclave{str(enclave_iterator)}_id""")
            text = text.replace(r"""ENCLAVE_NAME""",f"""ENCLAVE{str(enclave_iterator)}_NAME""")
            text = text.replace(r""" destroy_enclave""",f""" destroy_enclave_{str(enclave_iterator)}""")
            file.write(text)
            # if remote_attestation == 1 and (enclaveToAttest.__len__()>1 and str(enclave_iterator) in enclaveToAttest):
            #     file.write(Constants.___template_attest_enclave___)
            file.close()

    def writeXMLConf(filePath: str,enclave_iterator: int):
        """ Writes the XML config file for the enclave created(TO-DO:Implement a version for multiple enclaves)"""
        with open(filePath, "w") as f:
            logging.info("Writing xml.config for enclave"+str(enclave_iterator))
            f.write(Constants.__enclave_config_xml_code.replace("<ProdID></ProdID>","<ProdID>"+str(enclave_iterator-1)+"</ProdID>"))
            f.close()

    def writeLDS(filePath: str,enclave_iterator: int):
        """ Writes the enclave.lds file for manual compilation"""
        with open(filePath, "w") as f:
            logging.info("Writing enclave.lds")
            f.write(Constants.___enclave_lds_code.replace("Enclave1.so",f"libenclave{str(enclave_iterator)}.so"))
            f.close()

    def writeTestKey(filePath: str,enclave_iterator: int):
        """Creates RSA Private Key for signing the enclave, as a test"""
        with open(filePath, "w") as f:
            logging.info(f"Writing enclave_private_test.pem for enclave{str(enclave_iterator)}")
            f.write(Constants.___rsa_private_key__)
            f.close()

    def writeMakefileRA(filePath1: str, source:str,enclave_requested: int):
        """Creates the GNU Makefile for compilation of target"""
        with open(filePath1, "w") as f:
            logging.info("Writing makefile inside ./generated_trusted")
            f.write(Constants.___makefile_project_RA_header___)
            for s in source:
                f.write(s+""" """)
            application_app = r"""application/%.o: application/%.c"""
            App_Name = " $(App_Name): "
            app_objects = ""
            for i in range(1, enclave_requested + 1):
                application_app +=  f""" application/untrusted{str(i)}/enclave{str(i)}_u.c """
                App_Name +=  f"""application/untrusted{str(i)}/enclave{str(i)}_u.o """
                app_objects += Constants.__app_objects_template__.replace(r"""[i]""", f"{str(i)}")

            for i in range(1, enclave_requested + 1):
                App_Name += f"application/enclave_manager_{str(i)}.o "
            app_objects += f"""
application/enclave_manager_common.o: application/enclave_manager_common.c
	@echo "######## RUNNING $@ #########"	
	@$(CXX) $(App_Include_Paths) $(App_Link_Flags) -c $< -o $@
	@echo "GEN  =>  $@"
	"""
            App_Name += r"""application/enclave_manager_common.o remote_attestation/logfile.o remote_attestation/hexutil.o remote_attestation/msgio.o remote_attestation/common.o remote_attestation/client_starter.o""" + r""" $(App_Cpp_Objects)
	@$(CXX) $^ -o $@ $(App_Link_Flags)
	@echo "LINK =>  $@"  """
            application_app += r"""
	@$(CXX) $(SGX_COMMON_CXXFLAGS) $(App_Cpp_Flags) -c $< -o $@
	@echo "CXX  <=  $@" """

            final_text = Constants.___makefile_project_RA___.replace(r"""@replace_app_objects@""", app_objects)
            enclave_objects = ""
            enclave_targets = ""
            for i in range(1, enclave_requested + 1):
                enclave_targets +=  f"""enclave{str(i)}/$(Signed_Enclave_Name) """
                enclave_objects += Constants.__enclave_objects_template__.replace(r"""[i]""",f"{str(i)}")
            final_text = final_text.replace(r"""@replace_enclave_objects@""", enclave_objects).replace(
                r"""@replace_application_app@""", application_app).replace(r"""@replace_App_Name@""", App_Name)
            final_text = final_text.replace(r"""@replace_all_targets@""", enclave_targets)
            f.write(final_text)
            f.close()

    def writeMakefile(filePath1: str, source:str,enclave_requested:int):
        """Creates the GNU Makefile for compilation of target"""
        with open(filePath1, "w") as f:
            logging.info("Writing makefile inside ./generated_trusted")
            f.write(Constants.___makefile_project_header___)
            for s in source:
                f.write(s + " ")
            application_app = "application/%.o: application/%.c"
            App_Name = " $(App_Name): "
            app_objects = ""
            for i in range(1, enclave_requested+1):
                application_app += f" application/untrusted{str(i)}/enclave{str(i)}_u.c"
                App_Name += f"application/untrusted{str(i)}/enclave{str(i)}_u.o application/enclave_manager_{str(i)}.o "
                app_objects += Constants.__app_objects_template__.replace(r"""[i]""",f"{str(i)}")
            application_app += r"""
	@$(CXX) $(SGX_COMMON_CXXFLAGS) $(App_Cpp_Flags) -c $< -o $@
	@echo "CC  <=  $@" """
            App_Name += r""" $(App_Cpp_Objects)
	@$(CXX) $^ -o $@ $(App_Link_Flags)
	@echo "LINK =>  $@" """
            final_text = Constants.___makefile_project___ .replace(r"""@replace_app_objects@""",app_objects)
            enclave_objects = ""
            enclave_targets = ""
            for i in range(1, enclave_requested + 1):
                enclave_targets += f"enclave{str(i)}/$(Signed_Enclave_Name) "
                enclave_objects += Constants.__enclave_objects_template__.replace(r"""[i]""", f"{str(i)}")
            final_text = final_text.replace(r"""@replace_enclave_objects@""", enclave_objects).replace(r"""@replace_application_app@""",application_app).replace(r"""@replace_App_Name@""",App_Name)
            final_text = final_text.replace(r"""@replace_all_targets@""",enclave_targets)
            f.write(final_text)

            f.close()


    ___template_attest_enclave___ = r"""
int attest_enclave(sgx_enclave_id_t eid){
    int resultAttestation = -1;
    printf("Attestation starting, connecting with the server.");
    resultAttestation = perform_Attestation(eid);
    if (resultAttestation != 0)
    {
        printf("Error in attestation, aborting program.");
        abort();
    }
    printf("Attestation succeded, the enclave is valid. Starting the application...\n");
    return 0;
}


    """
    __enclave_manager_h_includes__ = r"""
#include <stdio.h>
#include <stdlib.h>
#include "enclave_u.h"

#ifdef _MSC_VER
#else
#include <sys/sysinfo.h>
#include <unistd.h>
#endif

#include "sgx_eid.h"
#include "sgx_urts.h"
#include "sgx_utils.h"
#include "sgx_capable.h"


#ifndef _ENCLAVE_MANAGER_
#define _ENCLAVE_MANAGER_


#ifdef _MSC_VER
#define ENCLAVE_NAME "Enclave.signed.dll"
#else
#define ENCLAVE_NAME "libenclave.signed.so"
#endif

#define check_hardware_capable 0

"""
    __enclave_manager_h_declarations__ = r"""
@prototype_attest_enclave@; //return type changed from void to int:my_enclave_id is static and cannot be changed in these functions
int destroy_enclave(void); //same
  
int query_and_enable_sgx_device(void);

static sgx_enclave_id_t my_enclave_id ="""
    __enclave_manager_c_code__ = r"""
    
#include "enclave_manager.h"
    
/* create the enclave, exit there is problems */
/* if the enclave already exists, just return and do nothing*/

@prototype_attest_enclave@{

	if(my_enclave_id != @replace_enclave_id@){
		return my_enclave_id;
	}

    int update = 0;
    sgx_launch_token_t token = {0};
    
#if check_hardware_capable
    if(query_and_enable_sgx_device()==-1){
        //compatibility problem
        //default error handling
        printf("Compatibility problem.\n");
        exit(-1);
    } 
#endif
    
    // load enclave
    sgx_status_t ret = sgx_create_enclave(ENCLAVE_NAME, SGX_DEBUG_FLAG, &token, &update, &my_enclave_id, NULL);
    if (ret != SGX_SUCCESS) {
        printf("App: error %#x, failed to create enclave,see Dev Guide for reference.\n", ret);
        exit(-2);
    }

    // succeed to load enclaves. 
    //printf("create_enclave() succeed, with id=%ld.\n",my_enclave_id);
    return (int)my_enclave_id;
}

/* return if ok, exit if error */ 
int destroy_enclave(void){
    if(my_enclave_id == @replace_enclave_id@)
        return 0;
    sgx_status_t ret = sgx_destroy_enclave(my_enclave_id);
    if(SGX_SUCCESS != ret){
        printf("App: error %#x, failed to destroy enclave,see Dev Guide for reference.\n", ret);
        exit(-1);
    }
	
	my_enclave_id = @replace_enclave_id@;
    //manage the error if needed
    return (int)my_enclave_id;
}


#if check_hardware_capable
/* name: query_and_enable_sgx_device()
 * Usage: 1. It detects SGX device's status inside target environment.
 *        2. And it will enable SGX device when it detects that SGX was set as Software Control inside BIOS. 
 *           So it can be used inside desktop application installer to enable SGX in target environment during installation.
 * Note: This function need be executed with Administrator privilege.
*/
int query_and_enable_sgx_device(void)
{
	sgx_device_status_t sgx_device_status;
	sgx_status_t sgx_ret = sgx_cap_get_status(&sgx_device_status);
	if (sgx_ret != SGX_SUCCESS) {
		printf("Failed to get SGX device status. If you have problem, run with administrator privilege\n");
		return -2;
	}
	else {
		switch (sgx_device_status) {
		case SGX_ENABLED:
			printf("SGX device has been enabled.\n");
            return 0; //ok
		
        case SGX_DISABLED_REBOOT_REQUIRED:
			printf("SGX device has been enabled. Please reboot your machine.\n");
			break;
		case SGX_DISABLED_LEGACY_OS:
			printf("SGX device can't be enabled on an OS that doesn't support EFI interface.\n");
			break;
		case SGX_DISABLED:
			printf("SGX is not enabled on this platform. More details are unavailable.\n");
			break;
		case SGX_DISABLED_MANUAL_ENABLE:
			printf("SGX device can be enabled manually in the BIOS setup.\n");
			break;
		case SGX_DISABLED_HYPERV_ENABLED:
			printf("Detected an unsupported version of Windows* 10 with Hyper-V enabled.\n");
			break;
		case SGX_DISABLED_UNSUPPORTED_CPU:
			printf("SGX is not supported by this CPU.\n");
			break;
		default:
			printf("Unexpected error.\n");
			break;

		case SGX_DISABLED_SCI_AVAILABLE:
			printf("SGX device can be enabled by a Software Control Interface, so enable it\n");
			sgx_ret = sgx_cap_enable_device(&sgx_device_status);
			if (sgx_ret != SGX_SUCCESS) {
				printf("Failed to Enable SGX device.\n");
			}
			switch (sgx_device_status)
			{
			case SGX_ENABLED:
				printf("SGX device has been enabled by Software Control Interface.\n");
				return 0; //ok

			case SGX_DISABLED_REBOOT_REQUIRED:
				printf("SGX device has been enabled by Software Control Interface. Please reboot your machine.\n");
				break;
			case SGX_DISABLED_LEGACY_OS:
				printf("SGX device can't be enabled on an OS that doesn't support EFI interface.\n");
				break;
			case SGX_DISABLED:
				printf("SGX is not enabled on this platform. More details are unavailable.\n");
				break;
			case SGX_DISABLED_MANUAL_ENABLE:
				printf("SGX device can be enabled manually in the BIOS setup.\n");
				break;
			case SGX_DISABLED_HYPERV_ENABLED:
				printf("Detected an unsupported version of Windows* 10 with Hyper-V enabled.\n");
				break;
			case SGX_DISABLED_UNSUPPORTED_CPU:
				printf("SGX is not supported by this CPU.\n");
				break;
			default:
				printf("Unexpected error.\n");
                break;
			}
			break;
		}
	}
	return -1;
}
#endif /* check_hardware_capable */
    """


    __enclave_config_xml_code = r"""
<EnclaveConfiguration>
    <ProdID></ProdID>
    <ISVSVN>1</ISVSVN>
    <StackMaxSize>0x40000</StackMaxSize>
    <HeapMaxSize>0x100000</HeapMaxSize>
    <TCSNum>5</TCSNum>
    <TCSMaxNum>5</TCSMaxNum>
	<TCSPolicy>1</TCSPolicy>
    <!--  Recommend changing 'DisableDebug' to 1 to make the enclave undebuggable for enclave release  -->
    <DisableDebug>0</DisableDebug>
    <MiscSelect>0</MiscSelect>
    <MiscMask>0xFFFFFFFF</MiscMask>
</EnclaveConfiguration>
    """

    # Linker script that hides the unnecessary symbols
    ___enclave_lds_code = r"""
    Enclave1.so
    {
        global:
            g_global_data_sim;
            g_global_data;
            enclave_entry;
            g_peak_heap_used;
        local:
            *;
    };
    """

    ___rsa_private_key__ = r"""
-----BEGIN RSA PRIVATE KEY-----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-----END RSA PRIVATE KEY-----
    """

    ___makefile_project_RA_header___ = r"""
# Copyright (C) 2011-2021 Intel Corporation. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of Intel Corporation nor the names of its
#     contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

######## SGX SDK Settings ########

SGX_SDK ?= ~/sgxsdk
SGX_MODE ?= HW
SGX_ARCH ?= x64
SGX_DEBUG ?= 1

ifeq ($(shell getconf LONG_BIT), 32)
	SGX_ARCH := x86
else ifeq ($(findstring -m32, $(CXXFLAGS)), -m32)
	SGX_ARCH := x86
endif

ifeq ($(SGX_ARCH), x86)
	SGX_COMMON_FLAGS := -m32
	SGX_LIBRARY_PATH := $(SGX_SDK)/lib
	SGX_ENCLAVE_SIGNER := $(SGX_SDK)/bin/x86/sgx_sign
	SGX_EDGER8R := $(SGX_SDK)/bin/x86/sgx_edger8r
else
	SGX_COMMON_FLAGS := -m64
	SGX_LIBRARY_PATH := $(SGX_SDK)/lib64
	SGX_ENCLAVE_SIGNER := $(SGX_SDK)/bin/x64/sgx_sign
	SGX_EDGER8R := $(SGX_SDK)/bin/x64/sgx_edger8r
endif

ifeq ($(SGX_DEBUG), 1)
ifeq ($(SGX_PRERELEASE), 1)
$(error Cannot set SGX_DEBUG and SGX_PRERELEASE at the same time!!)
endif
endif

ifeq ($(SGX_DEBUG), 1)
        SGX_COMMON_FLAGS += -O0 -g
else
        SGX_COMMON_FLAGS += -O2
endif

ifeq ($(SUPPLIED_KEY_DERIVATION), 1)
        SGX_COMMON_FLAGS += -DSUPPLIED_KEY_DERIVATION
endif

SGX_COMMON_FLAGS += -Wall -Wextra -Winit-self -Wpointer-arith -Wreturn-type \
                    -Waddress -Wsequence-point -Wformat-security \
                    -Wmissing-include-dirs -Wfloat-equal -Wundef -Wshadow \
                    -Wcast-align -Wconversion -Wredundant-decls -Wno-deprecated \
                    -Wno-unused-but-set-variable -Wno-unused-variable -Wno-conversion
SGX_COMMON_CFLAGS := $(SGX_COMMON_FLAGS) -Wjump-misses-init -Wstrict-prototypes -Wunsuffixed-float-constants 
SGX_COMMON_CXXFLAGS := $(SGX_COMMON_FLAGS) -Wnon-virtual-dtor -std=c++11

######## App Settings ########

ifneq ($(SGX_MODE), HW)
	Urts_Library_Name := sgx_urts_sim
else
	Urts_Library_Name := sgx_urts
endif

App_Cpp_Files := application/"""

    ___makefile_project_RA___ = r"""

App_Include_Paths := -I$(SGX_SDK)/include 

App_C_Flags := -fPIC -Wno-attributes $(App_Include_Paths)

# Three configuration modes - Debug, prerelease, release
#   Debug - Macro DEBUG enabled.
#   Prerelease - Macro NDEBUG and EDEBUG enabled.
#   Release - Macro NDEBUG enabled.
ifeq ($(SGX_DEBUG), 1)
        App_C_Flags += -DDEBUG -UNDEBUG -UEDEBUG
else ifeq ($(SGX_PRERELEASE), 1)
        App_C_Flags += -DNDEBUG -DEDEBUG -UDEBUG
else
        App_C_Flags += -DNDEBUG -UEDEBUG -UDEBUG
endif

App_Cpp_Flags := $(App_C_Flags)
App_Link_Flags := -L$(SGX_LIBRARY_PATH) -l$(Urts_Library_Name) -L. -lsgx_ukey_exchange -lpthread -Wl,-rpath=$(CURDIR)/sample_libcrypto -Wl,-rpath=$(CURDIR)

ifneq ($(SGX_MODE), HW)
	App_Link_Flags += -lsgx_epid_sim -lsgx_quote_ex_sim
else
	App_Link_Flags += -lsgx_epid -lsgx_quote_ex
endif

App_Cpp_Objects := $(App_Cpp_Files:.c=.o)

App_Name := app


######## Enclave Settings ########

ifneq ($(SGX_MODE), HW)
	Trts_Library_Name := sgx_trts_sim
	Service_Library_Name := sgx_tservice_sim
else
	Trts_Library_Name := sgx_trts
	Service_Library_Name := sgx_tservice
endif
Crypto_Library_Name := sgx_tcrypto

Enclave_Include_Paths := -I$(SGX_SDK)/include -I$(SGX_SDK)/include/tlibc -I$(SGX_SDK)/include/libcxx

Enclave_Files := @replace_enclave_needed@
Enclave_C_Flags := $(Enclave_Include_Paths) -nostdinc -fvisibility=hidden -fpie -ffunction-sections -fdata-sections
CC_BELOW_4_9 := $(shell expr "`$(CC) -dumpversion`" \< "4.9")
ifeq ($(CC_BELOW_4_9), 1)
	Enclave_C_Flags += -fstack-protector
else
	Enclave_C_Flags += -fstack-protector-strong
endif
Enclave_Cpp_Flags := $(Enclave_C_Flags) -nostdinc++

# Enable the security flags
Enclave_Security_Link_Flags := -Wl,-z,relro,-z,now,-z,noexecstack

# To generate a proper enclave, it is recommended to follow below guideline to link the trusted libraries:
#    1. Link sgx_trts with the `--whole-archive' and `--no-whole-archive' options,
#       so that the whole content of trts is included in the enclave.
#    2. For other libraries, you just need to pull the required symbols.
#       Use `--start-group' and `--end-group' to link these libraries.
# Do NOT move the libraries linked with `--start-group' and `--end-group' within `--whole-archive' and `--no-whole-archive' options.
# Otherwise, you may get some undesirable errors.
Enclave_Link_Flags := $(Enclave_Security_Link_Flags) \
    -Wl,--no-undefined -nostdlib -nodefaultlibs -nostartfiles -L$(SGX_LIBRARY_PATH) \
	-Wl,--whole-archive -l$(Trts_Library_Name) -Wl,--no-whole-archive \
	-Wl,--start-group -lsgx_tstdc -lsgx_tcxx -lsgx_tkey_exchange -l$(Crypto_Library_Name) -l$(Service_Library_Name) -Wl,--end-group \
	-Wl,-Bstatic -Wl,-Bsymbolic -Wl,--no-undefined \
	-Wl,-pie,-eenclave_entry -Wl,--export-dynamic  \
	-Wl,--defsym,__ImageBase=0 -Wl,--gc-sections   \
	-Wl,--version-script=enclave1/enclave.lds 	

Enclave_Cpp_Objects := $(Enclave_Cpp_Files:.c=.o)

####################################################
#ENCLAVE NAME AND FILES
#Actually configured for a Unix system, to change in
#case of Windows as OS 
####################################################


Enclave_Name := libenclave.so				#parameter to change if the system is on Windows to enclave.dll
Signed_Enclave_Name := libenclave1.signed.so		#ditto to enclave.signed.dll
	

ifeq ($(SGX_MODE), HW)
ifeq ($(SGX_DEBUG), 1)
	Build_Mode = HW_DEBUG
else ifeq ($(SGX_PRERELEASE), 1)
	Build_Mode = HW_PRERELEASE
else
	Build_Mode = HW_RELEASE
endif
else
ifeq ($(SGX_DEBUG), 1)
	Build_Mode = SIM_DEBUG
else ifeq ($(SGX_PRERELEASE), 1)
	Build_Mode = SIM_PRERELEASE
else
	Build_Mode = SIM_RELEASE
endif
endif


.PHONY: all run target
all: .config_$(Build_Mode)_$(SGX_ARCH)
	@$(MAKE) target

ifeq ($(Build_Mode), HW_RELEASE)
target: $(App_Name) $(Enclave_Name)
	@echo "The project has been built in release hardware mode."
	@echo "Please sign the $(Enclave_Name) first with your signing key before you run the $(App_Name) to launch and access the enclave."
	@echo "To sign the enclave use the command:"
	@echo "   $(SGX_ENCLAVE_SIGNER) sign -key <your key> -enclave $(Enclave_Name) -out <$(Signed_Enclave_Name)> -config $(Enclave_Config_File)"
	@echo "You can also sign the enclave using an external signing tool."
	@echo "To build the project in simulation mode set SGX_MODE=SIM. To build the project in prerelease mode set SGX_PRERELEASE=1 and SGX_MODE=HW."
else
target: $(App_Name) @replace_all_targets@
ifeq ($(Build_Mode), HW_DEBUG)
	@echo "The project has been built in debug hardware mode."
else ifeq ($(Build_Mode), SIM_DEBUG)
	@echo "The project has been built in debug simulation mode."
else ifeq ($(Build_Mode), HW_PRERELEASE)
	@echo "The project has been built in pre-release hardware mode."
else ifeq ($(Build_Mode), SIM_PRERELEASE)
	@echo "The project has been built in pre-release simulation mode."
else
	@echo "The project has been built in release simulation mode."
endif
endif

run: all
ifneq ($(Build_Mode), HW_RELEASE)
	@cd bin && $(CURDIR)/$(App_Name) 	
	@echo "RUN  =>  $(App_Name) [$(SGX_MODE)|$(SGX_ARCH), OK]"
endif

.config_$(Build_Mode)_$(SGX_ARCH):
	@rm -f .config_* $(App_Name) $(App_Name) $(Enclave_Name) $(Signed_Enclave_Name) $(App_Cpp_Objects) application/enclave_u.* $(Enclave_Cpp_Objects) enclave/enclave_t.* @touch .config_$(Build_Mode)_$(SGX_ARCH)


######## App Objects ########

@replace_app_objects@

remote_attestation/logfile.o: remote_attestation/logfile.c
	@$(CXX) -c $< -o $@
	@echo "GEN  <= $@"
remote_attestation/hexutil.o: remote_attestation/hexutil.c
	@$(CC) -c $< -o $@
	@echo "GEN  => $@"

remote_attestation/common.o: remote_attestation/common.cpp
	@$(CXX) $(App_Include_Paths) $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN  =>  $@"

remote_attestation/client_starter.o: remote_attestation/client_starter.cpp remote_attestation/common.o
	@echo "######## CREATING CLIENT_STARTER.O #########"
	@$(CXX) $(App_Include_Paths) $(App_Cpp_Flags) -Wno-deprecated -c $< -o $@
	@echo "GEN  => $@"

remote_attestation/msgio.o : remote_attestation/msgio.cpp
	@$(CXX) $(App_Include_Paths) $(App_Cpp_Flags) -c $< -o $@
	@echo "GEN  => $@"
	
@replace_application_app@
@replace_App_Name@

######## Enclave Objects ########

@replace_enclave_objects@

.PHONY: clean

clean:
	@rm -f .config_* $(App_Name) $(App_Name) $(Enclave_Name) $(Signed_Enclave_Name) $(App_Cpp_Objects) application/enclave_manager.o application/enclave_u.* $(Enclave_Cpp_Objects) enclave/enclave_t.*  remote_attestation/client_starter.o remote_attestation/common.o remote_attestation/msgio.o remote_attestation/hexutil.o  remote_attestation/logfile.o
    """""


    ___remoteAttestationFunctions_Header___ = r"""#define PSE_RETRIES	5	/* Arbitrary. Not too long, not too short. */
#ifndef __ENCLAVE_QUOTE_H
#define __ENCLAVE_QUOTE_H
#include "stdlib.h"
#include <string.h>
#include <sgx_utils.h>
#ifdef _WIN32
#include <sgx_tae_service.h>
#endif
#include <sgx_tkey_exchange.h>
#include <sgx_tcrypto.h>
#include "sys/types.h"
#include <sgx_spinlock.h>
typedef enum _ra_state
{
    ra_inited= 0,
    ra_get_gaed,
    ra_proc_msg2ed
}ra_state;
typedef struct _ra_db_item_t
{
    sgx_ec256_public_t          g_a;
    sgx_ec256_public_t          g_b;
    sgx_ec_key_128bit_t         vk_key;
    sgx_ec256_public_t          sp_pubkey;
    sgx_ec256_private_t         a;
    sgx_ps_sec_prop_desc_t      ps_sec_prop;
    sgx_ec_key_128bit_t         mk_key;
    sgx_ec_key_128bit_t         sk_key;
    sgx_ec_key_128bit_t         smk_key;
    sgx_quote_nonce_t           quote_nonce;
    sgx_target_info_t           qe_target;
    ra_state                    state;
    sgx_spinlock_t              item_lock;
    uintptr_t                   derive_key_cb;
} ra_db_item_t;
#endif
//PUBLIC KEY ENCODED IN THE ENCLAVE
static const sgx_ec256_public_t def_service_public_key = {
    {
        0x72, 0x12, 0x8a, 0x7a, 0x17, 0x52, 0x6e, 0xbf,
        0x85, 0xd0, 0x3a, 0x62, 0x37, 0x30, 0xae, 0xad,
        0x3e, 0x3d, 0xaa, 0xee, 0x9c, 0x60, 0x73, 0x1d,
        0xb0, 0x5b, 0xe8, 0x62, 0x1c, 0x4b, 0xeb, 0x38
    },
    {
        0xd4, 0x81, 0x40, 0xd9, 0x50, 0xe2, 0x57, 0x7b,
        0x26, 0xee, 0xb7, 0x41, 0xe7, 0xc6, 0x14, 0xe2,
        0x24, 0xb7, 0xbd, 0xc9, 0x03, 0xf2, 0x9a, 0x28,
        0xa8, 0x3c, 0xc8, 0x10, 0x11, 0x14, 0x5e, 0x06
    }
};"""

    ___remoteAttestationFunctions_Implementations___ = r"""/*----------------------------------------------------------------------
 * WARNING
 *----------------------------------------------------------------------
 *
 * End developers should not normally be calling these functions
 * directly when doing remote attestation:
 *
 *    sgx_get_ps_sec_prop()
 *    sgx_get_quote()
 *    sgx_get_quote_size()
 *    sgx_get_report()
 *    sgx_init_quote()
 *
 * These functions short-circuits the RA process in order
 * to generate an enclave quote directly!
 *
 * The high-level functions provided for remote attestation take
 * care of the low-level details of quote generation for you:
 *
 *   sgx_ra_init()
 *   sgx_ra_get_msg1
 *   sgx_ra_proc_msg2
 *
 *----------------------------------------------------------------------
 */
sgx_status_t get_report(sgx_report_t *report, sgx_target_info_t *target_info)
{
#ifdef SGX_HW_SIM
	return sgx_create_report(NULL, NULL, report);
#else
	return sgx_create_report(target_info, NULL, report);
#endif
}
#ifdef _WIN32
size_t get_pse_manifest_size ()
{
	return sizeof(sgx_ps_sec_prop_desc_t);
}
sgx_status_t get_pse_manifest(char *buf, size_t sz)
{
	sgx_ps_sec_prop_desc_t ps_sec_prop_desc;
	sgx_status_t status= SGX_ERROR_SERVICE_UNAVAILABLE;
	int retries= PSE_RETRIES;
	do {
		status= sgx_create_pse_session();
		if ( status != SGX_SUCCESS ) return status;
	} while (status == SGX_ERROR_BUSY && retries--);
	if ( status != SGX_SUCCESS ) return status;
	status= sgx_get_ps_sec_prop(&ps_sec_prop_desc);
	if ( status != SGX_SUCCESS ) return status;
	memcpy(buf, &ps_sec_prop_desc, sizeof(ps_sec_prop_desc));
	sgx_close_pse_session();
	return status;
}
#endif
sgx_status_t enclave_ra_init(sgx_ec256_public_t key, int b_pse,
	sgx_ra_context_t *ctx, sgx_status_t *pse_status)
{
	sgx_status_t ra_status;
	/*
	 * If we want platform services, we must create a PSE session 
	 * before calling sgx_ra_init()
	 */
#ifdef _WIN32
	if ( b_pse ) {
		int retries= PSE_RETRIES;
		do {
			*pse_status= sgx_create_pse_session();
			if ( *pse_status != SGX_SUCCESS ) return SGX_ERROR_UNEXPECTED;
		} while (*pse_status == SGX_ERROR_BUSY && retries--);
		if ( *pse_status != SGX_SUCCESS ) return SGX_ERROR_UNEXPECTED;
	}
	ra_status= sgx_ra_init(&key, b_pse, ctx);
	if ( b_pse ) {
		int retries= PSE_RETRIES;
		do {
			*pse_status= sgx_close_pse_session();
			if ( *pse_status != SGX_SUCCESS ) return SGX_ERROR_UNEXPECTED;
		} while (*pse_status == SGX_ERROR_BUSY && retries--);
		if ( *pse_status != SGX_SUCCESS ) return SGX_ERROR_UNEXPECTED;
	}
#else
	ra_status= sgx_ra_init(&key, 0, ctx);
#endif
	return ra_status;
}
sgx_status_t enclave_ra_init_def(int b_pse, sgx_ra_context_t *ctx,
	sgx_status_t *pse_status)
{
	return enclave_ra_init(def_service_public_key, b_pse, ctx, pse_status);
}
/*
 * Return a SHA256 hash of the requested key. KEYS SHOULD NEVER BE
 * SENT OUTSIDE THE ENCLAVE IN PLAIN TEXT. This function let's us
 * get proof of possession of the key without exposing it to untrusted
 * memory.
 */
sgx_status_t enclave_ra_get_key_hash(sgx_status_t *get_keys_ret,
	sgx_ra_context_t ctx, sgx_ra_key_type_t type, sgx_sha256_hash_t *hash)
{
	sgx_status_t sha_ret;
	sgx_ra_key_128_t k;
	// First get the requested key which is one of:
	//  * SGX_RA_KEY_MK 
	//  * SGX_RA_KEY_SK
	// per sgx_ra_get_keys().
	*get_keys_ret= sgx_ra_get_keys(ctx, type, &k);
	if ( *get_keys_ret != SGX_SUCCESS ) return *get_keys_ret;
	/* Now generate a SHA hash */
	sha_ret= sgx_sha256_msg((const uint8_t *) &k, sizeof(k), 
		(sgx_sha256_hash_t *) hash); // Sigh.
	/* Let's be thorough */
	memset(k, 0, sizeof(k));
	return sha_ret;
}
sgx_status_t enclave_ra_close(sgx_ra_context_t ctx)
{
        sgx_status_t ret;
        ret = sgx_ra_close(ctx);
        return ret;
}
    """

    ___makefile_project_header___ = r"""
# Copyright (C) 2011-2021 Intel Corporation. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of Intel Corporation nor the names of its
#     contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#
######## SGX SDK Settings ########
SGX_SDK ?= ~/sgxsdk
SGX_MODE ?= HW
SGX_ARCH ?= x64
SGX_DEBUG ?= 1
ifeq ($(shell getconf LONG_BIT), 32)
	SGX_ARCH := x86
else ifeq ($(findstring -m32, $(CXXFLAGS)), -m32)
	SGX_ARCH := x86
endif
ifeq ($(SGX_ARCH), x86)
	SGX_COMMON_FLAGS := -m32
	SGX_LIBRARY_PATH := $(SGX_SDK)/lib
	SGX_ENCLAVE_SIGNER := $(SGX_SDK)/bin/x86/sgx_sign
	SGX_EDGER8R := $(SGX_SDK)/bin/x86/sgx_edger8r
else
	SGX_COMMON_FLAGS := -m64
	SGX_LIBRARY_PATH := $(SGX_SDK)/lib64
	SGX_ENCLAVE_SIGNER := $(SGX_SDK)/bin/x64/sgx_sign
	SGX_EDGER8R := $(SGX_SDK)/bin/x64/sgx_edger8r
endif
ifeq ($(SGX_DEBUG), 1)
ifeq ($(SGX_PRERELEASE), 1)
$(error Cannot set SGX_DEBUG and SGX_PRERELEASE at the same time!!)
endif
endif
ifeq ($(SGX_DEBUG), 1)
        SGX_COMMON_FLAGS += -O0 -g
else
        SGX_COMMON_FLAGS += -O2
endif
ifeq ($(SUPPLIED_KEY_DERIVATION), 1)
        SGX_COMMON_FLAGS += -DSUPPLIED_KEY_DERIVATION
endif
SGX_COMMON_FLAGS += -Wall -Wextra -Winit-self -Wpointer-arith -Wreturn-type \
                    -Waddress -Wsequence-point -Wformat-security \
                    -Wmissing-include-dirs -Wfloat-equal -Wundef -Wshadow \
                    -Wcast-align -Wconversion -Wredundant-decls -Wno-deprecated \
                    -Wno-unused-but-set-variable -Wno-unused-variable -Wno-conversion
SGX_COMMON_CFLAGS := $(SGX_COMMON_FLAGS) -Wjump-misses-init -Wstrict-prototypes -Wunsuffixed-float-constants 
SGX_COMMON_CXXFLAGS := $(SGX_COMMON_FLAGS) -Wnon-virtual-dtor -std=c++11
######## App Settings ########
ifneq ($(SGX_MODE), HW)
	Urts_Library_Name := sgx_urts_sim
else
	Urts_Library_Name := sgx_urts
endif
App_Cpp_Files := application/"""

    ___makefile_project___ = r"""
App_Include_Paths := -I$(SGX_SDK)/include 
App_C_Flags := -fPIC -Wno-attributes $(App_Include_Paths)
# Three configuration modes - Debug, prerelease, release
#   Debug - Macro DEBUG enabled.
#   Prerelease - Macro NDEBUG and EDEBUG enabled.
#   Release - Macro NDEBUG enabled.
ifeq ($(SGX_DEBUG), 1)
        App_C_Flags += -DDEBUG -UNDEBUG -UEDEBUG
else ifeq ($(SGX_PRERELEASE), 1)
        App_C_Flags += -DNDEBUG -DEDEBUG -UDEBUG
else
        App_C_Flags += -DNDEBUG -UEDEBUG -UDEBUG
endif
App_Cpp_Flags := $(App_C_Flags)
App_Link_Flags := -L$(SGX_LIBRARY_PATH) -l$(Urts_Library_Name) -L. -lsgx_ukey_exchange -lpthread -Wl,-rpath=$(CURDIR)/sample_libcrypto -Wl,-rpath=$(CURDIR)
ifneq ($(SGX_MODE), HW)
	App_Link_Flags += -lsgx_epid_sim -lsgx_quote_ex_sim
else
	App_Link_Flags += -lsgx_epid -lsgx_quote_ex
endif
App_Cpp_Objects := $(App_Cpp_Files:.c=.o)
App_Name := app
######## Enclave Settings ########
ifneq ($(SGX_MODE), HW)
	Trts_Library_Name := sgx_trts_sim
	Service_Library_Name := sgx_tservice_sim
else
	Trts_Library_Name := sgx_trts
	Service_Library_Name := sgx_tservice
endif
Crypto_Library_Name := sgx_tcrypto
Enclave_Cpp_Files := enclave/enclave.c
Enclave_Include_Paths := -I$(SGX_SDK)/include -I$(SGX_SDK)/include/tlibc -I$(SGX_SDK)/include/libcxx
Enclave_C_Flags := $(Enclave_Include_Paths) -nostdinc -fvisibility=hidden -fpie -ffunction-sections -fdata-sections
CC_BELOW_4_9 := $(shell expr "`$(CC) -dumpversion`" \< "4.9")
ifeq ($(CC_BELOW_4_9), 1)
	Enclave_C_Flags += -fstack-protector
else
	Enclave_C_Flags += -fstack-protector-strong
endif
Enclave_Cpp_Flags := $(Enclave_C_Flags) -nostdinc++
# Enable the security flags
Enclave_Security_Link_Flags := -Wl,-z,relro,-z,now,-z,noexecstack
# To generate a proper enclave, it is recommended to follow below guideline to link the trusted libraries:
#    1. Link sgx_trts with the `--whole-archive' and `--no-whole-archive' options,
#       so that the whole content of trts is included in the enclave.
#    2. For other libraries, you just need to pull the required symbols.
#       Use `--start-group' and `--end-group' to link these libraries.
# Do NOT move the libraries linked with `--start-group' and `--end-group' within `--whole-archive' and `--no-whole-archive' options.
# Otherwise, you may get some undesirable errors.
Enclave_Link_Flags := $(Enclave_Security_Link_Flags) \
    -Wl,--no-undefined -nostdlib -nodefaultlibs -nostartfiles -L$(SGX_LIBRARY_PATH) \
	-Wl,--whole-archive -l$(Trts_Library_Name) -Wl,--no-whole-archive \
	-Wl,--start-group -lsgx_tstdc -lsgx_tcxx -lsgx_tkey_exchange -l$(Crypto_Library_Name) -l$(Service_Library_Name) -Wl,--end-group \
	-Wl,-Bstatic -Wl,-Bsymbolic -Wl,--no-undefined \
	-Wl,-pie,-eenclave_entry -Wl,--export-dynamic  \
	-Wl,--defsym,__ImageBase=0 -Wl,--gc-sections   \
	-Wl,--version-script=enclave1/enclave.lds 	 
Enclave_Cpp_Objects := $(Enclave_Cpp_Files:.c=.o)
####################################################
#ENCLAVE NAME AND FILES
#Actually configured for a Unix system, to change in
#case of Windows as OS 
####################################################
Enclave_Name := libenclave.so				#parameter to change if the system is on Windows to enclave.dll
Signed_Enclave_Name := libenclave.signed.so		#ditto to enclave.signed.dll
ifeq ($(SGX_MODE), HW)
ifeq ($(SGX_DEBUG), 1)
	Build_Mode = HW_DEBUG
else ifeq ($(SGX_PRERELEASE), 1)
	Build_Mode = HW_PRERELEASE
else
	Build_Mode = HW_RELEASE
endif
else
ifeq ($(SGX_DEBUG), 1)
	Build_Mode = SIM_DEBUG
else ifeq ($(SGX_PRERELEASE), 1)
	Build_Mode = SIM_PRERELEASE
else
	Build_Mode = SIM_RELEASE
endif
endif
.PHONY: all run target
all: .config_$(Build_Mode)_$(SGX_ARCH)
	@$(MAKE) target
ifeq ($(Build_Mode), HW_RELEASE)
target: $(App_Name) @replace_all_targets@
	@echo "The project has been built in release hardware mode."
	@echo "Please sign the $(Enclave_Name) first with your signing key before you run the $(App_Name) to launch and access the enclave."
	@echo "To sign the enclave use the command:"
	@echo "   $(SGX_ENCLAVE_SIGNER) sign -key <your key> -enclave $(Enclave_Name) -out <$(Signed_Enclave_Name)> -config $(Enclave_Config_File)"
	@echo "You can also sign the enclave using an external signing tool."
	@echo "To build the project in simulation mode set SGX_MODE=SIM. To build the project in prerelease mode set SGX_PRERELEASE=1 and SGX_MODE=HW."
else
target: $(App_Name) @replace_all_targets@
ifeq ($(Build_Mode), HW_DEBUG)
	@echo "The project has been built in debug hardware mode."
else ifeq ($(Build_Mode), SIM_DEBUG)
	@echo "The project has been built in debug simulation mode."
else ifeq ($(Build_Mode), HW_PRERELEASE)
	@echo "The project has been built in pre-release hardware mode."
else ifeq ($(Build_Mode), SIM_PRERELEASE)
	@echo "The project has been built in pre-release simulation mode."
else
	@echo "The project has been built in release simulation mode."
endif
endif
run: all
ifneq ($(Build_Mode), HW_RELEASE)
	@cd bin && $(CURDIR)/$(App_Name) 	
	@echo "RUN  =>  $(App_Name) [$(SGX_MODE)|$(SGX_ARCH), OK]"
endif
.config_$(Build_Mode)_$(SGX_ARCH):
	@rm -f .config_* $(App_Name) $(App_Name) $(Enclave_Name) $(Signed_Enclave_Name) $(App_Cpp_Objects) application/enclave[i]_u.* $(Enclave_Cpp_Objects) enclave[i]/enclave[i]_t.* @touch .config_$(Build_Mode)_$(SGX_ARCH)
######## App Objects ########
@replace_app_objects@
@replace_application_app@
@replace_App_Name@
######## Enclave Objects ########
@replace_enclave_objects@
.PHONY: clean
clean:
	@rm -f .config_* $(App_Name) $(App_Name) $(Enclave_Name) $(Signed_Enclave_Name) $(App_Cpp_Objects) application/enclave_manager.o application/enclave_u.* $(Enclave_Cpp_Objects) enclave/enclave_t.*
    """

    __app_objects_template__ = r"""
application/untrusted[i]/enclave[i]_u.h: $(SGX_EDGER8R) enclave[i]/enclave[i].edl
	@cd application && $(SGX_EDGER8R) --untrusted ../enclave[i]/enclave[i].edl --use-prefix --search-path ./enclave[i] --search-path $(SGX_SDK)/include --untrusted-dir untrusted[i]/
	@echo "GEN  =>  $@"
	
application/untrusted[i]/enclave[i]_u.c: application/untrusted[i]/enclave[i]_u.h
	
application/untrusted[i]/enclave[i]_u.o: application/untrusted[i]/enclave[i]_u.c
	@$(CC) $(SGX_COMMON_CFLAGS) $(App_C_Flags)  -c $< -o $@
	@echo "CC   <=  $@"
application/enclave_manager_[i].o: application/enclave_manager_[i].c
	@echo "######## RUNNING $@ #########"	
	@$(CXX) $(App_Include_Paths) $(App_Link_Flags) -c $< -o $@
	@echo "GEN  =>  $@"
	
	"""
    __enclave_objects_template__ = r"""
enclave[i]/enclave[i]_t.h: $(SGX_EDGER8R) enclave[i]/enclave[i].edl
	@cd enclave[i] && $(SGX_EDGER8R) --trusted enclave[i].edl --search-path $(SGX_SDK)/include 
	@echo "GEN  =>  $@" 
enclave[i]/enclave[i]_t.c: enclave[i]/enclave[i]_t.h
enclave[i]/enclave[i]_t.o: enclave[i]/enclave[i]_t.c
	@$(CC) $(SGX_COMMON_CFLAGS) $(Enclave_C_Flags) -c $< -o $@
	@echo "CC   <=  $<"
enclave[i]/enclave.o: enclave[i]/enclave.c enclave[i]/enclave[i]_t.h
	@$(CC) $(SGX_COMMON_CFLAGS) $(Enclave_C_Flags) -c $< -o $@
	@echo "CC  <=  $<"
enclave[i]/$(Enclave_Name): enclave[i]/enclave[i]_t.o enclave[i]/enclave.o
	@$(CC) $^ -o $@ $(Enclave_Link_Flags) 
	@echo "LINK =>  $@"
enclave[i]/$(Signed_Enclave_Name): enclave[i]/$(Enclave_Name)
	@$(SGX_ENCLAVE_SIGNER) sign -key enclave[i]/enclave_private_test.pem -enclave enclave[i]/$(Enclave_Name) -out $@ -config enclave[i]/enclave.config.xml
	@echo "SIGN =>  $@"
	@mv enclave[i]/$(Signed_Enclave_Name) libenclave[i].signed.so
"""



