import subprocess
import logging
from enum import Enum
import sys
import regex

def isToolAvailable():
    from shutil import which
    logging.debug("checking ctags availability")
    if which("ctags") is None:
        raise Exception("Error #C001: Error ctags is not available")	


class c_types(Enum):
    enum =  'g'
    union =  'u'
    struct =  's'
    function = 'f'
    define = 'd'

def getGlobalVars(filePATH:str):
    """ Call ctags for a single file filepath and return the output, only global variables are returned"""

    res = subprocess.check_output(['ctags','-x','--c-types=v',filePATH])
    tags: str = res.decode("utf-8")
    vars_global = regex.findall(r"^[\w\d]+",tags,regex.MULTILINE)
    return vars_global

def getFunctionsInFile(filepath:str):
    """ call ctags for a single file and return the output, only NON static functions are returned
    retu example = {
    'add': {
            'filepath': 'file_c/file1.c', 
            'signature': '(int x, int y)'}, 
    'ciaone': {
            'filepath': 'file_c/file1.c', cc
            'signature': '()'}, 
    """
    res = subprocess.check_output(['ctags','-f','-', "--file-scope=no", '--fields=a-f-i-k-K-l-m-n-s+S-z-t', '--c-types=f',filepath])
    tags: str = res.decode("utf-8")
    lines = tags.split("\r\n")
    lines.pop() # remove last empty element
    
    retu = {}
    for line in lines:
        f = line.split("\t")+ ["signature:()"]
        signature: str = f[3].split(":")[1]
        retu[f[0]] = {"filepath": f[1], "signature": signature}
    return retu


# deprecated
def getFunctionsInDir(dirpath):
    """ call ctags for multiple file in a dir and return the output. only NON static functions are returned
    retu example = [
    ['foo',  'file_c\\file1.c', '/^int foo(char *buf, size_t len)$/;"', 'signature:(char *buf, size_t len)'], 
    ['foo2', 'file_c\\file2.c', '/^int foo2(char *buf, size_t len)$/;"', 'signature:(char *buf, size_t len)'], 
    ['main', 'file_c\\file1.c', '/^int main() {$/'], 
    ['main2','file_c\\file2.c', '/^int main2() {$/']
    ]"""
    # here we use a pipe, because it is requested to read directory and files with pipe or file
    # manual : http://ctags.sourceforge.net/ctags.html 
    p1 = subprocess.Popen(["echo", dirpath], stdout=subprocess.PIPE) 
    p2 = subprocess.Popen(["ctags", "-f", "-", "--c-types=f",  "--recurse", "--file-scope=no",  '--fields=a-f-i-k-K-l-m-n-s+S-z-t',"-L", "-"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
    output, _err = p2.communicate()
    
    tags: str = output.decode("utf-8")
    lines = tags.replace('\r', '').split("\n")
    
    retu = [] # here I split the lined according to the output generated
    for line in lines:
        retu.append(line.split("\t"))
    retu.pop() # remove last empty element
    return retu



def getNamesOfType(filepaths: [], type: c_types)-> {}:
    """assert that all names defined in all files are different and return all the names
    return["name"]="filepath" """
    #in order to revert, it's filepaths = "'" + "\n".join(filepaths) + "'"
    if sys.platform.startswith('linux'):
        filepaths =  "" + "\n".join(filepaths) + ""
    elif sys.platform.startswith('win32'):
        filepaths =  "'" + "\n".join(filepaths) + "'"
    p1 = subprocess.Popen(["printf", filepaths], stdout=subprocess.PIPE) 
    #p2 = subprocess.Popen(["ctags", "-f", "-", "--c-types="+type.value, "--file-scope=no",  '--fields=a-f-i-k-K-l-m-n-s-S-z-t',"-L", "-"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["ctags", "-f", "-", "--c-types="+type.value,  '--fields=a-f-i-k-K-l-m-n-s-S-z-t',"-L", "-"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
    output, _err = p2.communicate()
    
    
    
    
    
    tags: str = output.decode("utf-8")
    lines = tags.replace('\r', '').split("\n")
    lines.pop() # remove empty line

    seen = {} #the entire line
    for line in lines:
        logging.debug(line)
        name, filePath, _deff = line.split("\t")
        if name in seen:
            raise Exception(f"""Error #C002: assert unique definition {name}. multiple definition are found in files {filePath}, {seen[name]}  """) 
        else:
            seen[name] = filePath
    return seen



#try yourself 


#print(getFunctionsInFile("file_c/file1.c"))
#print(getNamesOfType(["file_c/file1.c", "file_c/file2.c", "file_c/file1.h"], c_types.function))
