import subprocess,os,sys
import pydot
from pydot import Dot, Edge
import logging

def isToolAvailable():
    from shutil import which
    logging.debug("checking frama-c availability")
    if which("frama-c") is None:
        raise Exception("Error #D001: frama-c is not available")

def getEdgesRecursive(functionName, filePath):
    """
        get a list of [{source: functName1, dest: funcName2}] of function reached by the self function
    """
    try:
        cmd = os.system('frama-c -cg tmp.dot -lib-entry -main '+ functionName + ' '+ filePath)
        #res = subprocess.check_call(["frama-c","-cg", "tmp.dot","-lib-entry", "-main" ,functionName, filePath],cwd=os.curdir) #use this for debug
        
        #subprocess.check_output(['frama-c','-cg','tmp.dot','-lib-entry','-main',functionName,filePath])
        dotList = pydot.graph_from_dot_file("tmp.dot")

        functionNameEdges = []
        for dotObj in dotList:
            dotObj: Dot
            for edge in dotObj.get_edges():
                edge: Edge
                #print(edge.get_source().split(" ")[1] + " -> " + edge.get_destination().split(" ")[1]) #only for debug purpose
                functionNameEdges.append({"source": edge.get_source().split(" ")[1], "dest": edge.get_destination().split(" ")[1]})
        os.system('rm tmp.dot')
        return functionNameEdges
    except FileNotFoundError:
        raise Exception("Error #D002: the function: " + functionName + " was not found in file: " + filePath)
        
    except subprocess.CalledProcessError as grepexc:   
        if grepexc.returncode == 5:
            #print("The caller has no callee")
            return []
        elif grepexc.returncode == 1:
            raise Exception("Error #D003: Frama-c tells about errors in or with file: " + filePath)
        else:
            #print (str(grepexc.returncode) + grepexc.output)
            raise grepexc


def getFunctionNamesUsedByFunction(functionName, filePath, recursive:bool):
    """
        return a list of name: string.  @return ['foo', 'printf']
    """
    try:
        cmd = os.system('frama-c -cg tmp.dot -lib-entry -main '+ functionName + ' '+ filePath)
        dotList = pydot.graph_from_dot_file("tmp.dot")
        
        functionNameList = []
        for dotObj in dotList:
            dotObj: Dot
            for edge in dotObj.get_edges():
                edge: Edge
                #print(edge.get_source().split(" ")[1] + " -> " + edge.get_destination().split(" ")[1]) #only for debug purpose
                if recursive or (edge.get_destination().split(" ")[1] == functionName):
                    functionNameList.append(edge.get_source().split(" ")[1])
        
        os.system('rm tmp.dot')
        return functionNameList
    except FileNotFoundError:
        raise Exception("Error #D002: the function: " + functionName + " was not found in file: " + filepath)
        
    except subprocess.CalledProcessError as grepexc:   
        if grepexc.returncode == 5:
            #pprint("The caller has no callee")
            return []
        elif grepexc.returncode == 1:
            raise Exception("Error #D003: Errors in or with file: " + filepath)
        else:
            #print (str(grepexc.returncode) + grepexc.output)
            raise grepexc

                                                                                                          
        
    



#pprint(getFunctionNamesUsedByFunction("add", "file_c/file1.c", True))
#TODO logging
