import subprocess
import logging
import re
import os

def isToolAvailable():
    from shutil import which
    logging.debug("checking gcc availability")
    if which("gcc") is None:
        raise Exception("Error #G001: GCC is not available")
    configureTool()


def configureTool():
    sgx_configuration_file = "./include/configuration_file.txt"
    if os.path.exists(sgx_configuration_file) and os.path.isfile(sgx_configuration_file):
        return
    else:
        subprocess.run(['gcc', '-nostdlib', '-nostartfiles' ,'-I./include/tlibc', '-I./include/', '-I./include/tlibc/sys', '-aux-info', sgx_configuration_file, './include/__convert_tool_configure.c'])

    if os.path.exists(sgx_configuration_file) and os.path.isfile(sgx_configuration_file):
        raise Exception(f"Error #G002: can't configure correctly gcc sgx libraries, re-run to check, remove also all non sgx function from file {sgx_configuration_file} ")



# remove all comments from source code
def removeCommentsFromSourceCode(filepath:str):
    try:
        logging.debug("preprocessing file with: gcc -fpreprocessed -dD -E " + filepath )
        res = subprocess.check_output(['gcc', '-fpreprocessed', '-dD', '-E',filepath])
        text: str = res.decode("utf-8")
        return re.sub(r'#[\s]+[\d]*[\s]+"[^"]*"', '', text)  # remove the comments lines and return
        
    except Exception:
        raise Exception("Error #G003: with file in: " + filepath)
        

# try yourself
#print(removeCommentsFromSourceCode("file_c/file1.c"))


def getHeadersWhereFunctionsAreDef(functionNames: [], filepaths: [])-> [] :
    """functions is an array of function object
    the function are searched in sgx libraries, not in std ones. if a function is not found raise an exception
    return example{'memcpy': 'string.h', 'really_mult': 'file1.h'} """

    
    logging.debug("Compute libraries for functions: " + str(functionNames))
    
    try:
        sgx_configuration_file = "../../include/configuration_file.txt"
        with open(sgx_configuration_file, 'r', encoding='utf8') as tmp1:
            gccOutput = tmp1.read()
            #.decode("utf-8")
            #print(gccOutput)# for debug

        sgx_libraries_file = "../../include/__convert_tool_configure.c"
        with open(sgx_libraries_file, 'r', encoding='utf8') as tmp1:
            sgx_libraries = re.findall(r"#include[\s]*\<([\w.]*)\>", tmp1.read(), flags=re.MULTILINE)

        includePath = {}
        for funcionName in functionNames:
            pattern = r"\/\* (?:[\w.]*\/)*([\w]*\.h):[\d]*:.. \*\/[\w\s*]*[\s*]+" + funcionName + r"[^;]+;"
            match = re.findall(pattern, gccOutput, flags=re.MULTILINE)
            if match:
                includePath[funcionName] = match[0]
            else:
                if "__" not in funcionName: # esclude the macros with __
                    raise Exception(f"Error #G004: the function: '{funcionName}' is not found in sgx library. In enclave functions you must use only those or user defined ones")
                
                #frama-c pattern for libraries
                pattern = r"__locale_([\w]*)_ptr"
                match = re.search(pattern, funcionName)

                if match: #try to search the library
                    include = match.group(1) + '.h'

                    if include in sgx_libraries:
                        includePath[funcionName] = include
                        logging.warning(f"Limitation on Frama-c, some functions are not available for library {include}, pay attenction on function used by ecall in this library ")
                    else:
                        raise Exception (f"can't find function {funcionName}, please run frama-c interface with debug option to get more information")


                



        return includePath

    except FileNotFoundError:
        raise Exception ("Error #G005: missing configuration file, please run the configuration in gccInterface")
    


#f = ["printf", "atoi"]
#paths = ["pacchetti_codice_c/banner/banner.c"]
#print(getHeadersWhereFunctionsAreDef(f, paths))