from cFileReaderWriter import cFileReaderWriter
from Function import Function
import logging
import regex
from constants import Constants

class CodeRA(object):

    ___requestClientAttestationImplementationCode_raw___ = r""" return startServerIAS();"""
    ___requestClientAttestationEntireCode_raw___ = r"""int requestClientAttestation()
{
    return startServerIAS();
}"""

    ___serverAttestationImplementationCode___ = r"""int result_client_attestation = requestClientAttestation();
      if(result_client_attestation != 0){
        printf("Attestation failed, closing socket.\n");
        return 1;
    }"""
    @staticmethod
    def searchForAttestation(cFile:cFileReaderWriter):
        """search in the code for pragma client_attestation or server_attestation"""
        logging.debug("search of pragma attestation definitions")

        requests = regex.findall(r"#\s*pragma\s*attest_enclave_\d+", cFile.contents, flags=regex.MULTILINE)
        if requests:
            return 1
        else:
            requests = regex.findall(r"#\s*pragma\s*move_to_server[^\_E\d+]", cFile.contents, flags=regex.MULTILINE)
        if requests:
            return 1
        requests = regex.findall(r"#\s*pragma\s*SERVER_ATTESTATION\n",cFile.contents,flags=regex.MULTILINE)
        if requests:
            return 2
        return 0

    def getOrderFunctions(cFile:cFileReaderWriter,ecallFunctions):
        """It returns the order of the ECall and attest functions called inside the function main"""
        orderFuncs = []
        mainF = cFile.getFunction("main")
        results = regex.findall("(\w+\()| (attest_enclave_\d)", mainF.implementationCode,flags=regex.MULTILINE)  # checks inside the source code what functions are called
        for r in results:
            if (r[0][:len(r[0]) - 1] in ecallFunctions.keys()):
                orderFuncs.append(ecallFunctions[r[0][:len(r[0]) - 1]].askedforEnclave[0])
            elif (r[0][0:len(r[0]) - 1] == "attest_enclave_"):
                orderFuncs.append("A" + str(r[0][len(r[0]) - 1]))
            elif (r[1][:len(r[1]) - 1] == "attest_enclave_"):
                orderFuncs.append("A" + str(r[1][len(r[1]) - 1]))

        return orderFuncs

    def generateClientCode(cFile:cFileReaderWriter,f:Function, enclaveToAttest: str,ecallFunctions):
        """It receives the function f that has asked for an implementation of remote attestation, and returns it modified with the call to the RA functions"""
        funcClient=Function("", f.returnType, f.functionName, f.arguments,f.implementationCode, f.entireCode, f.filePath)
        funcClient.implementationCode = funcClient.implementationCode.replace(f"#pragma attest_enclave_{enclaveToAttest}",f"""sgx_enclave_id_t eid{enclaveToAttest} = create_enclave_{enclaveToAttest}();
        attest_enclave(eid{enclaveToAttest});""")
        #funcClient.implementationCode = funcClient.implementationCode.replace(f"#pragma attest_enclave_{enclaveToAttest}",f"""sgx_enclave_id_t eid{enclaveToAttest} = create_enclave_{enclaveToAttest}();
 # attest_enclave(eid{enclaveToAttest});""")
        funcClient.entireCode = f"""{funcClient.returnType} {funcClient.functionName} {funcClient.arguments} {funcClient.implementationCode}"""
        cFile.replaceFunction(f,funcClient)



    def generateServerCode(cFile:cFileReaderWriter,f:Function):
        """It receives the function that has asked for an implementation of remote attestation, and returns it modified with the call of the RA functions"""
        funcServer = Function("",f.returnType,f.functionName,f.arguments,f.implementationCode,f.entireCode,f.filePath)
        funcServer.implementationCode = funcServer.implementationCode.replace("#pragma SERVER_ATTESTATION",CodeRA.___serverAttestationImplementationCode___)
        funcServer.entireCode = f"""{funcServer.returnType} {funcServer.functionName} {funcServer.arguments} {funcServer.implementationCode}"""
        cFile.replaceFunction(f,funcServer)



    def insertFunctionOfClientAttestation(cFile:cFileReaderWriter,filepath:str):
        """ Creates the function requestClientAttestation for the server and add to the cFileReaderWriter struct """
        newFunc = Function("","int","requestClientAttestation","()",CodeRA.___requestClientAttestationImplementationCode_raw___,
                           CodeRA.___requestClientAttestationEntireCode_raw___,filepath)
        cFile.addFunction(newFunc)

    @staticmethod
    def configSPSettings(signerSPID:str,iasPrimaryKey:str,iasSecondaryKey:str,iasReportFilePath:str):
        logging.debug("Writing the parameters inside settings.h for the service provider")
        with open('../server_material/settings.h', 'r') as file:
            filedata = file.read()

        filedata = filedata.replace('#spid_signer#', '"' + signerSPID + '"')
        filedata = filedata.replace('#prim_key#', '"' + iasPrimaryKey + '"')
        filedata = filedata.replace('#sec_key#', '"' + iasSecondaryKey + '"')
        filedata = filedata.replace('#rep_file#', '"' + iasReportFilePath + '"')

        with open('../server_material/settings.h', 'w') as file:
            file.write(filedata)

    def updateMakefileServerProvider(enclaves_requested:int):
        """ Updates the makefile for the service provider, generating all the lines """
        with open("../Makefile", "r") as f:
            file_data = f.read()
            application_app = r"""application/$(App_Name).o: application/$(App_Name).c """
            App_Name = " $(App_Name): "
            app_objects = ""
            enclave_objects = ""
            enclave_targets = ""
            for i in range(1, enclaves_requested + 1):
                application_app += f"application/untrusted{i}/enclave{i}_u.c"
                App_Name +=  f"application/untrusted{str(i)}/enclave{str(i)}_u.o "
                app_objects += Constants.__app_objects_template__.replace(r"""[i]""", r"""""" + str(i))
                enclave_targets += f"enclave{i}/$(Signed_Enclave_Name) "
                enclave_objects += Constants.__enclave_objects_template__.replace(r"""[i]""",r""""""+str(i))

            #we perform two cycles because we need first all enclavesX_u.o files and then enclave_manager_X.o files, otherwise it won't compile
            for i in range(1, enclaves_requested + 1):
                App_Name += f"application/enclave_manager_{str(i)}.o "

            App_Name += r""" server_material/server_interface.o server_material/agent_wget.o server_material/iasrequest.o server_material/enclave_verify.o server_material/byteorder.o server_material/common.o server_material/crypto.o server_material/hexutil.o server_material/fileio.o server_material/base64.o server_material/msgio.o server_material/agent_curl.o"""
            App_Name += r""" application/$(App_Name).o 
	@$(CXX) -g $^ -o $@ $(App_Link_Flags) -lcurl -lcrypto
	@echo "LINK => $@" """
            application_app +=  r""" 
	@$(CXX) -g -Wno-unused-parameter $(SGX_COMMON_CXXFLAGS) $(App_Cpp_Flags) -c $< -o $@
	@echo "CXX <= $<" """

            file_data = file_data.replace(r"""@replace_enclave_objects@""",enclave_objects).replace(r"""@replace_application_app""",application_app).replace(r"""@replace_App_Name@""",App_Name)
            file_data = file_data.replace(r"""@replace_app_objects@""", app_objects).replace(r"""@replace_all_targets@""",enclave_targets)

        with open("../Makefile", "w") as fw:
            fw.write(file_data)
            f.close()


    def setServerAddress(address:str):
        """ It inserts the address to which the client connect in config.h """
        with open("../remote_attestation/client_starter.cpp",'r') as file:
            filedata=file.read()

        if address == "":
            filedata = filedata.replace('@replace_address_server@','localhost')
        else:
            filedata = filedata.replace('@replace_address_server@',address)

        with open('../remote_attestation/client_starter.cpp','w') as file:
            file.write(filedata)

   # def
